@@ -1,4 +1,4 @@
-/* $Id: base1.c,v 1.103 2002/09/07 13:34:38 karim Exp $
+/* $Id: base1.c,v 1.123 2003/02/05 10:46:49 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -148,7 +148,7 @@ checkbid(GEN bid)
 void
 checkprimeid(GEN id)
 {
-  if (typ(id) != t_VEC || lg(id) != 6)
+  if (typ(id) != t_VEC || lg(id) != 6 || typ(id[2]) != t_COL)
     err(talker,"incorrect prime ideal");
 }
 
@@ -263,12 +263,10 @@ transroot(GEN x, int i, int j)
   k=x[i]; x[i]=x[j]; x[j]=k; return x;
 }
 
-#define randshift (BITS_IN_RANDOM - 3)
-
 GEN
 tschirnhaus(GEN x)
 {
-  gpmem_t av = avma, av2;
+  pari_sp av = avma, av2;
   long a, v = varn(x);
   GEN u, p1 = cgetg(5,t_POL);
 
@@ -278,17 +276,16 @@ tschirnhaus(GEN x)
   p1[1] = evalsigne(1)|evalvarn(0)|evallgef(5);
   do
   {
-    a = mymyrand() >> randshift; if (a==0) a =1; p1[4]=lstoi(a);
-    a = mymyrand() >> (randshift-1); if (a>=4) a-=8; p1[3]=lstoi(a);
-    a = mymyrand() >> (randshift-1); if (a>=4) a-=8; p1[2]=lstoi(a);
-    u = caract2(x,p1,v); av2=avma;
+    a = random_bits(2); if (a==0) a  = 1; p1[4] = lstoi(a);
+    a = random_bits(3); if (a>=4) a -= 8; p1[3] = lstoi(a);
+    a = random_bits(3); if (a>=4) a -= 8; p1[2] = lstoi(a);
+    u = caract2(x,p1,v); av2 = avma;
   }
   while (lgef(srgcd(u,derivpol(u))) > 3); /* while u not separable */
   if (DEBUGLEVEL>1)
     fprintferr("Tschirnhaus transform. New pol: %Z",u);
   avma=av2; return gerepileupto(av,u);
 }
-#undef randshift
 
 int
 gpolcomp(GEN p1, GEN p2)
@@ -387,7 +384,7 @@ _res(long n, long s, long k)
 GEN
 galois(GEN x, long prec)
 {
-  gpmem_t av = avma, av1;
+  pari_sp av = avma, av1;
   long i,j,k,n,f,l,l2,e,e1,pr,ind;
   GEN x1,p1,p2,p3,p4,p5,w,z,ee;
   static int ind5[20]={2,5,3,4, 1,3,4,5, 1,5,2,4, 1,2,3,5, 1,4,2,3};
@@ -634,7 +631,7 @@ galois(GEN x, long prec)
 GEN
 galoisapply(GEN nf, GEN aut, GEN x)
 {
-  gpmem_t av=avma,tetpil;
+  pari_sp av=avma,tetpil;
   long lx,j,N;
   GEN p,p1,y,pol;
 
@@ -718,7 +715,7 @@ get_nfpol(GEN x, GEN *nf)
 static GEN
 nfiso0(GEN a, GEN b, long fliso)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long n,m,i,vb,lx;
   GEN nfa,nfb,p1,y,la,lb;
 
@@ -810,7 +807,7 @@ nfisincl(GEN a, GEN b) { return nfiso0(a,b,0); }
 GEN
 get_roots(GEN x,long r1,long prec)
 {
-  GEN roo = (typ(x)==t_VEC)? dummycopy(x): roots(x,prec);
+  GEN roo = (typ(x)!=t_POL)? dummycopy(x): roots(x,prec);
   long i, ru = (lg(roo)-1 + r1) >> 1;
 
   for (i=1; i<=r1; i++) roo[i]=lreal((GEN)roo[i]);
@@ -838,7 +835,7 @@ quicktrace(GEN x, GEN sym)
 static GEN
 trace_col(GEN x, GEN T)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN t = gzero;
   long i, l = lg(x);
 
@@ -969,7 +966,7 @@ get_mul_table(GEN x,GEN basden,GEN invbas)
   for (i=1; i<=n; i++)
     for (j=i; j<=n; j++)
     {
-      gpmem_t av = avma;
+      pari_sp av = avma;
       z = gres(gmul((GEN)bas[j],(GEN)bas[i]), x);
       z = mulmat_pol(invbas, z); /* integral column */
       if (den)
@@ -991,7 +988,7 @@ make_Tr(GEN x, GEN w)
   GEN sym = cgetg(n+2,t_VEC);
   GEN den = cgetg(n+1,t_VEC);
   GEN T = cgetg(n+1,t_MAT);
-  gpmem_t av;
+  pari_sp av;
 
   sym = polsym(x, n-1);
   p1 = get_bas_den(w);
@@ -1230,7 +1227,7 @@ GEN
 nffromhnfbasis(GEN nf, GEN x)
 {
   long tx = typ(x);
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN u;
   if (!is_vec_t(tx)) return gcopy(x);
   nf = checknf(nf);
@@ -1242,7 +1239,7 @@ GEN
 nftohnfbasis(GEN nf, GEN x)
 {
   long tx = typ(x);
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN u;
   if (!is_vec_t(tx)) return gcopy(x);
   nf = checknf(nf);
@@ -1254,11 +1251,11 @@ static GEN
 get_red_G(nfbasic_t *T, GEN *pro)
 {
   GEN G, u, u0 = NULL;
-  gpmem_t av;
+  pari_sp av;
   long i, prec, extraprec, n = degpol(T->x);
   nffp_t F;
 
-  extraprec = (long) (0.5 * n * sizeof(long) / 8.);
+  extraprec = (long) (0.25 * n * (sizeof(long) / 4));
   prec = DEFAULTPREC + extraprec;
   nffp_init(&F, T, *pro, prec);
   av = avma;
@@ -1294,7 +1291,7 @@ get_LLL_basis(nfbasic_t *T, GEN *pro)
   if (T->r1 != degpol(T->x)) u = get_red_G(T, pro);
   else
   {
-    u = lllint_marked(1, make_Tr(T->x, T->bas), 100, 1, &u,NULL,NULL);
+    u = lllfp_marked(1, make_Tr(T->x, T->bas), 100, 0, DEFAULTPREC, 1);
     if (!u) u = idmat(1);
   }
   return u;
@@ -1395,7 +1392,7 @@ nfpolred(int part, nfbasic_t *T)
   rev = modreverse_i(phi, x);
   for (i=1; i<=n; i++) a[i] = (long)RX_RXQ_compo((GEN)a[i], rev, xbest);
   mat = vecpol_to_mat(Q_remove_denom(a, &d), n);
-  if (!is_pm1(d)) mat = gdiv(hnfmodid(mat,d), d); else mat = idmat(n);
+  if (d) mat = gdiv(hnfmodid(mat,d), d); else mat = idmat(n);
 
   (void)carrecomplet(diviiexact(dxbest,T->dK), &(T->index));
   T->bas= mat_to_vecpol(mat, v);
@@ -1406,7 +1403,7 @@ nfpolred(int part, nfbasic_t *T)
 GEN
 get_nfindex(GEN bas)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long n = lg(bas)-1;
   GEN d, mat = vecpol_to_mat(Q_remove_denom(bas, &d), n);
   if (!d) { avma = av; return gun; }
@@ -1471,7 +1468,7 @@ nfbasic_init(GEN x, long flag, GEN fa, nfbasic_t *T)
 GEN
 _initalg(GEN x, long flag, long prec)
 {
-  const gpmem_t av = avma;
+  const pari_sp av = avma;
   GEN nf, rev = NULL, ro = NULL;
   nfbasic_t T;
 
@@ -1540,18 +1537,15 @@ nfgetprec(GEN x)
   return (typ(ro)==t_VEC)? precision((GEN)ro[1]): DEFAULTPREC;
 }
 
-GEN
-nfnewprec(GEN nf, long prec)
+/* assume nf is an nf */
+static GEN
+_nfnewprec(GEN nf, long prec)
 {
-  gpmem_t av = avma;
-  GEN NF;
+  pari_sp av = avma;
+  GEN NF = dummycopy(nf);
   nffp_t F;
 
-  if (typ(nf) != t_VEC) err(talker,"incorrect nf in nfnewprec");
-  if (lg(nf) == 11) return bnfnewprec(nf,prec);
-  if (lg(nf) ==  7) return bnrnewprec(nf,prec);
   (void)checknf(nf);
-  NF = dummycopy(nf);
   NF[5] = (long)dummycopy((GEN)NF[5]);
   remake_GM(NF, &F, prec);
   NF[6]  = (long)F.ro;
@@ -1560,6 +1554,28 @@ nfnewprec(GEN nf, long prec)
   return gerepilecopy(av, NF);
 }
 
+GEN
+nfnewprec(GEN nf, long prec)
+{
+  long l = lg(nf);
+  GEN z, res = NULL;
+
+  if (typ(nf) != t_VEC) err(talker,"incorrect nf in nfnewprec");
+  if (l == 3) {
+    res = cgetg(3, t_VEC);
+    res[2] = lcopy((GEN)nf[2]);
+    nf = (GEN)nf[1]; l = lg(nf);
+  }
+  switch(l)
+  {
+    case 11: z = bnfnewprec(nf,prec); break;
+    case  7: z = bnrnewprec(nf,prec); break;
+    default: z = _nfnewprec(nf,prec); break;
+  }
+  if (res) res[1] = (long)z; else res = z;
+  return res;
+}
+
 /********************************************************************/
 /**                                                                **/
 /**                           POLRED                               **/
@@ -1570,7 +1586,7 @@ static void
 remove_duplicates(GEN y, GEN a)
 {
   long k, i, l = lg(y);
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN z;
 
   if (l < 2) return;
@@ -1636,7 +1652,7 @@ allpolred(GEN x, long flag, GEN fa, GEN *pta, FP_chk_fun *CHECK)
 GEN
 polred0(GEN x, long flag, GEN fa)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN y, a;
   int fl = 0;
 
@@ -1655,7 +1671,7 @@ polred0(GEN x, long flag, GEN fa)
 GEN
 ordred(GEN x)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN y;
 
   if (typ(x) != t_POL) err(typeer,"ordred");
@@ -1743,35 +1759,48 @@ get_polmin(CG_data *d, GEN x)
 static GEN
 chk_gen(void *data, GEN x)
 {
-  gpmem_t av = avma;
-  GEN g = get_polchar((CG_data*)data,x);
-  GEN h = modulargcd(g,derivpol(g));
+  pari_sp av = avma, av1;
+  GEN h, g = get_polchar((CG_data*)data,x);
+  av1 = avma;
+  h = modulargcd(g,derivpol(g));
   if (degpol(h)) { avma = av; return NULL; }
   if (DEBUGLEVEL>3) fprintferr("  generator: %Z\n",g);
-  return g;
+  avma = av1; return gerepileupto(av, g);
 }
 
-/* mat = base change matrix, gram = LLL-reduced T2 matrix */
 static GEN
-chk_gen_init(FP_chk_fun *chk, GEN gram, GEN mat)
+norm_ei_from_Q(GEN Q, long k)
+{
+  GEN s = gcoeff(Q, k, k);
+  long i;
+  for (i = 1; i < k; i++)
+    s = gadd(s, gmul(gcoeff(Q,i,i), gsqr(gcoeff(Q,i,k))));
+  return s;
+}
+
+/* mat = base change matrix, r = Cholesky form of the quadratic form [matrix
+ * Q from algo 2.7.6] */
+static GEN
+chk_gen_init(FP_chk_fun *chk, GEN r, GEN mat)
 {
   CG_data *d = (CG_data*)chk->data;
   GEN P,bound,prev,x,B;
-  long l = lg(gram), N = l-1,i,prec,prec2;
+  long l = lg(r), N = l-1,i,prec;
   int skipfirst = 0;
 
   d->u = mat;
-  d->ZKembed = gmul(d->M,   mat);
+  d->ZKembed = gmul(d->M, mat);
 
   bound = d->bound;
   prev = NULL;
   x = zerocol(N);
   for (i=1; i<l; i++)
   {
-    B = gcoeff(gram,i,i);
+    B = norm_ei_from_Q(r, i);
     if (gcmp(B,bound) >= 0) continue; /* don't assume increasing norms */
 
-    x[i] = un; P = get_polmin(d,x);
+    /* use canon_pol to recognized trivial automorphism P --> P(-x) */
+    x[i] = un; P = get_polmin(d,x); (void)canon_pol(P);
     x[i] = zero;
     if (degpol(P) == N)
     {
@@ -1784,9 +1813,9 @@ chk_gen_init(FP_chk_fun *chk, GEN gram, GEN mat)
     /* Q(w[1],...,w[i-1]) = Q[X]/(prev) is a strict subfield of nf */
     if (prev && degpol(prev) < N && !gegal(prev,P))
     {
-      if (degpol(prev) * degpol(P) > 64) continue; /* too expensive */
+      if (degpol(prev) * degpol(P) > 128) continue; /* too expensive */
       P = compositum(prev,P);
-      P = (GEN)P[lg(P)-1];
+      P = (GEN)P[lg(P)-1]; (void)canon_pol(P);
       if (degpol(P) == N) continue;
       if (DEBUGLEVEL>2 && degpol(P) != degpol(prev))
         fprintferr("chk_gen_init: subfield %Z\n",P);
@@ -1797,35 +1826,32 @@ chk_gen_init(FP_chk_fun *chk, GEN gram, GEN mat)
   chk->skipfirst = skipfirst;
   if (DEBUGLEVEL>2) fprintferr("chk_gen_init: skipfirst = %ld\n",skipfirst);
 
-  /* should be gexpo( max_k C^n_k (bound/k)^(k/2) ) */
-  prec2 = (1 + (((gexpo(bound)*N)/2) >> TWOPOTBITS_IN_LONG));
-  if (prec2 < 0) prec2 = 0;
-  prec = 3 + prec2;
+  /* should be DEF + gexpo( max_k C^n_k (bound/k)^(k/2) ) */
+  prec = DEFAULTPREC + (((gexpo(bound)*N)/2) >> TWOPOTBITS_IN_LONG);
   if (DEBUGLEVEL)
     fprintferr("chk_gen_init: new prec = %ld (initially %ld)\n", prec, d->prec);
-  if (prec > d->prec) err(bugparier, "precision problem in polredabs");
+  if (prec > d->prec) err(bugparier, "polredabs (precision problem)");
   if (prec < d->prec) d->ZKembed = gprec_w(d->ZKembed, prec);
   return bound;
 }
 
-/* store phi(beta mod z). Suitable for gerepileupto */
+/* store phi(beta mod z). */
 static GEN
-storeeval(GEN beta, GEN z, GEN lead)
+storeeval(GEN a, GEN x, GEN z, GEN lead)
 {
-  GEN y = cgetg(3,t_VEC);
-  z = gcopy(z);
-  beta = lead? gdiv(beta, lead): gcopy(beta);
+  GEN y, beta = modreverse_i(a, x);
+  if (lead) beta = gdiv(beta, lead);
+  y = cgetg(3,t_VEC);
   y[1] = (long)z;
-  y[2] = (long)to_polmod(beta,z);
-  return y;
+  y[2] = (long)to_polmod(beta,z); return y;
 }
 
 static GEN
 storeraw(GEN beta, GEN z)
 {
   GEN y = cgetg(3,t_VEC);
-  y[1] = lcopy(z);
-  y[2] = lcopy(beta); return y;
+  y[1] = (long)z;
+  y[2] = (long)beta; return y;
 }
 
 static void
@@ -1855,23 +1881,20 @@ findmindisc(GEN *py, GEN *pa)
 static GEN
 storepol(GEN x, GEN z, GEN a, GEN lead, long flag)
 {
-  GEN y, b;
+  GEN y;
   if (flag & nf_RAW)
     y = storeraw(a, z);
   else if (flag & nf_ORIG)
-  {
-    b = modreverse_i(a, x);
-    y = storeeval(b,z, lead);
-  }
+    y = storeeval(a, x, z, lead);
   else
-    y = gcopy(z);
+    y = z;
   return y;
 }
 
 static GEN
 storeallpol(GEN x, GEN z, GEN a, GEN lead, long flag)
 {
-  GEN y, b;
+  GEN y;
 
   if (flag & nf_RAW)
   {
@@ -1882,15 +1905,11 @@ storeallpol(GEN x, GEN z, GEN a, GEN lead, long flag)
   else if (flag & nf_ORIG)
   {
     long i, c = lg(z);
-    b = cgetg(c, t_VEC);
-    for (i=1; i<c; i++)
-      b[i] = (long)modreverse_i((GEN)a[i], x);
-
     y = cgetg(c,t_VEC);
-    for (i=1; i<c; i++) y[i] = (long)storeeval((GEN)b[i], (GEN)z[i], lead);
+    for (i=1; i<c; i++) y[i] = (long)storeeval((GEN)a[i], x, (GEN)z[i], lead);
   }
   else
-    y = gcopy(z);
+    y = z;
   return y;
 }
 
@@ -1922,7 +1941,7 @@ _polredabs(nfbasic_t *T, GEN *u)
     d.M    = F.M;
     if (R)
     {
-      v = fincke_pohst(_vec(R),NULL,5000,1, 0, &chk);
+      v = fincke_pohst(_vec(R),NULL,200,1, 0, &chk);
       if (v) break;
     }
     if (i==MAXITERPOL) err(accurer,"polredabs0");
@@ -1936,8 +1955,8 @@ _polredabs(nfbasic_t *T, GEN *u)
 GEN
 polredabs0(GEN x, long flag)
 {
+  pari_sp av = avma;
   long i, l, vx;
-  gpmem_t av = avma;
   GEN y, a, u;
   nfbasic_t T;
 
@@ -1970,15 +1989,25 @@ polredabs0(GEN x, long flag)
   if (flag & nf_ALL)
   {
     if (u) for (i=1; i<l; i++) a[i] = lmul(T.bas, gmul(u, (GEN)a[i]));
-    y = storeallpol(x,y,a,T.lead,flag);
+    y = storeallpol(x, y, a, T.lead, flag);
+    if (flag & nf_ADDZK) err(impl,"nf_ADDZK flag when nf_ALL set (polredabs)");
   }
   else
   {
-    findmindisc(&y, &a);
+    GEN z;
+    findmindisc(&y, &a); z = y;
     if (u) a = gmul(T.bas, gmul(u, a));
-    y = storepol(x,y,a,T.lead,flag);
+    y = storepol(x, y, a, T.lead, flag);
+    if (flag & nf_ADDZK)
+    {
+      GEN t, y0 = y, B = vecpol_to_mat(T.bas, lg(T.bas)-1);
+      t = (flag & nf_ORIG)? lift_intern((GEN)y[2]): modreverse_i(a, x);
+      y = cgetg(3, t_VEC);
+      y[1] = (long)y0;
+      y[2] = lmul(RXQ_powers(t, z, degpol(z)-1), B);
+    }
   }
-  return gerepileupto(av, y);
+  return gerepilecopy(av, y);
 }
 
 GEN
@@ -2032,7 +2061,7 @@ trivroots(GEN nf)
 GEN
 rootsof1(GEN nf)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long N,k,i,ws,prec;
   GEN p1,y,d,list,w;
 
@@ -2079,7 +2108,7 @@ static GEN
 dirzetak0(GEN nf, long N0)
 {
   GEN vect,p1,pol,disc,c,c2;
-  gpmem_t av=avma;
+  pari_sp av=avma;
   long i,j,k,limk,lx;
   ulong q,p,rem;
   byteptr d=diffptr;
@@ -2092,9 +2121,10 @@ dirzetak0(GEN nf, long N0)
   c2[1]=c[1]=1; for (i=2; i<=N0; i++) c[i]=0;
   court[2] = 0;
 
+  maxprime_check((ulong)N0);
   while (court[2]<=N0)
   {
-    NEXT_PRIME_VIADIFF_CHECK(court[2], d);
+    NEXT_PRIME_VIADIFF(court[2], d);
     if (smodis(disc,court[2])) /* court does not divide index */
       { vect = (GEN) simplefactmod(pol,court)[1]; lx=lg(vect); }
     else
@@ -2112,7 +2142,7 @@ dirzetak0(GEN nf, long N0)
         while (q<=(ulong)N0)
         {
           for (k=1; k<=limk; k++) c2[k*q] += c[k];
-          q = smulss(q,p,&rem);
+          q = umuluu(q,p,&rem);
           if (rem) break;
           limk /= p;
         }
@@ -2149,7 +2179,7 @@ initzeta(GEN pol, long prec)
   GEN limx,bnf,resi,zet,C,coeflog,racpi,aij,tabj,colzero, *tabcstn, *tabcstni;
   GEN c_even,ck_even,c_odd,ck_odd,serie_even,serie_odd,serie_exp,Pi;
   long N0,imin,imax,r1,r2,ru,R,N,i,j,k,n;
-  gpmem_t av,av2,tetpil;
+  pari_sp av,av2,tetpil;
   long court[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3),0};
   stackzone *zone, *zone0, *zone1;
 
@@ -2311,7 +2341,7 @@ initzeta(GEN pol, long prec)
       t[1] = lstoi(coef[i]); t[2] = lmul((GEN)t[1],p1);
       for (j=2; j<=ru; j++)
       {
-        gpmem_t av2 = avma;
+        pari_sp av2 = avma;
         p2 = gmul((GEN)t[j], p1);
         t[j+1] = (long)gerepileuptoleaf(av2, divrs(p2,j));
       }
@@ -2381,7 +2411,7 @@ gzetakall(GEN nfz, GEN s, long flag, long prec2)
   GEN lambd,gammas,gammaunmoins,gammas2,gammaunmoins2,var1,var2;
   GEN p1,unmoins,gexpro,gar,val,valm,valk,valkm;
   long ts = typ(s), r1,r2,ru,imax,i,j,k,N0,sl,prec,bigprec;
-  gpmem_t av = avma;
+  pari_sp av = avma;
 
   if (typ(nfz)!=t_VEC || lg(nfz)!=10 || typ(nfz[1]) != t_VEC)
     err(talker,"not a zeta number field in zetakall");
