@@ -1,4 +1,4 @@
-/* $Id: gen2.c,v 1.53 2002/09/11 22:29:31 karim Exp $
+/* $Id: gen2.c,v 1.59 2003/01/19 10:22:53 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -32,31 +32,31 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 void
 gop1z(GEN (*f)(GEN), GEN x, GEN y)
 {
-  gpmem_t av=avma; gaffect(f(x), y); avma=av;
+  pari_sp av=avma; gaffect(f(x), y); avma=av;
 }
 
 void
 gop2z(GEN (*f)(GEN, GEN), GEN x, GEN y, GEN z)
 {
-  gpmem_t av=avma; gaffect(f(x, y), z); avma=av;
+  pari_sp av=avma; gaffect(f(x, y), z); avma=av;
 }
 
 void
 gops2gsz(GEN (*f)(GEN, long), GEN x, long s, GEN z)
 {
-  gpmem_t av=avma; gaffect(f(x, s), z); avma=av;
+  pari_sp av=avma; gaffect(f(x, s), z); avma=av;
 }
 
 void
 gops2sgz(GEN (*f)(long, GEN), long s, GEN y, GEN z)
 {
-  gpmem_t av=avma; gaffect(f(s, y), z); avma=av;
+  pari_sp av=avma; gaffect(f(s, y), z); avma=av;
 }
 
 void
 gops2ssz(GEN (*f)(long, long), long s, long y, GEN z)
 {
-  gpmem_t av=avma; gaffect(f(s, y), z); avma=av;
+  pari_sp av=avma; gaffect(f(s, y), z); avma=av;
 }
 
 /*******************************************************************/
@@ -89,14 +89,14 @@ opgs2(int (*f)(GEN, GEN), GEN y, long s)
 void
 gopsg2z(GEN (*f)(GEN, GEN), long s, GEN y, GEN z)
 {
-  gpmem_t av=avma;
+  pari_sp av=avma;
   affsi(s,court_p); gaffect(f(court_p,y),z); avma=av;
 }
 
 void
 gopgs2z(GEN (*f)(GEN, GEN), GEN y, long s, GEN z)
 {
-  gpmem_t av=avma;
+  pari_sp av=avma;
   affsi(s,court_p); gaffect(f(y,court_p),z); avma=av;
 }
 
@@ -172,15 +172,14 @@ matsize(GEN x)
 
 /*******************************************************************/
 /*                                                                 */
-/*                            GREFFE                               */
-/*              Greffe d'une serie sur un polynome                 */
+/*                  Conversion t_POL --> t_SER                     */
 /*                                                                 */
 /*******************************************************************/
 
 GEN
 greffe(GEN x, long l, long use_stack)
 {
-  long i,e,k,vx;
+  long i, e, k;
   GEN y;
 
   if (typ(x)!=t_POL) err(notpoler,"greffe");
@@ -192,16 +191,16 @@ greffe(GEN x, long l, long use_stack)
   }
   if (gcmp0(x))
   {
-    y[1]=evalvalp(l-2) | evalvarn(varn(x));
-    i=2; while(i<l) { y[i]=x[2]; i++; }
+    y[1] = evalvalp(l-2) | evalvarn(varn(x));
+    for (i = 2; i < l; i++) y[i] = zero;
     return y;
   }
-  vx=varn(x); e=gval(x,vx);
-  y[1]=evalsigne(1) | evalvalp(e) | evalvarn(vx);
-  k=lgef(x)-e-1; i=l-1;
-  if (k<l)
-    while (i>k) { y[i]=zero; i--; }
-  while (i>=2) { y[i]=x[i+e]; i--; }
+  
+  e = polvaluation(x, NULL);
+  y[1] = evalsigne(1) | evalvalp(e) | evalvarn(varn(x));
+  k = lgef(x)-1 - e;
+  for (i = l-1; i >  k; i--) y[i] = zero;
+  for (       ; i >= 2; i--) y[i] = x[i+e];
   return y;
 }
 
@@ -215,7 +214,7 @@ long
 gtolong(GEN x)
 {
   long y, tx=typ(x);
-  gpmem_t av=avma;
+  pari_sp av=avma;
 
   switch(tx)
   {
@@ -337,7 +336,7 @@ gcmp1(GEN x)
 int
 gcmp_1(GEN x)
 {
-  gpmem_t av;
+  pari_sp av;
   long l,y;
   GEN p1;
 
@@ -386,10 +385,9 @@ gcmp_1(GEN x)
 int
 gcmp(GEN x, GEN y)
 {
-  long tx, ty, f;
-  gpmem_t av;
+  long tx = typ(x), ty = typ(y), f;
+  pari_sp av;
 
-  tx=typ(x); ty=typ(y);
   if (is_intreal_t(tx))
     { if (is_intreal_t(ty)) return mpcmp(x,y); }
   else
@@ -401,7 +399,11 @@ gcmp(GEN x, GEN y)
       return f > 0? 1
                   : f? -1: 0;
     }
-    if (!is_frac_t(tx)) err(typeer,"comparison");
+    if (!is_frac_t(tx)) 
+    {
+      if (ty == t_STR) return -1;
+      err(typeer,"comparison");
+    }
   }
   if (ty == t_STR) return -1;
   if (!is_intreal_t(ty) && !is_frac_t(ty)) err(typeer,"comparison");
@@ -516,8 +518,8 @@ gegal_try(GEN x, GEN y)
 int
 gegal(GEN x, GEN y)
 {
-  ulong tx;
-  gpmem_t av;
+  pari_sp av;
+  long tx;
   long i;
   
   if (x == y) return 1;
@@ -618,7 +620,7 @@ long
 ggval(GEN x, GEN p)
 {
   long tx=typ(x), tp=typ(p), vx, v, i, val;
-  gpmem_t av, limit;
+  pari_sp av, limit;
   GEN p1,p2;
 
   if (isexactzero(x)) return VERYBIGINT;
@@ -711,7 +713,7 @@ long
 pvaluation(GEN x, GEN p, GEN *py)
 {
   long v;
-  gpmem_t av, av2;
+  pari_sp av, av2;
   GEN p1,p2;
 
   if (egalii(p,gdeux))
@@ -883,7 +885,7 @@ GEN
 gabs(GEN x, long prec)
 {
   long tx=typ(x), lx, i;
-  gpmem_t av, tetpil;
+  pari_sp av, tetpil;
   GEN y,p1;
 
   switch(tx)
@@ -1110,7 +1112,7 @@ void
 gaffect(GEN x, GEN y)
 {
   long i, j, k, l, v, vy, lx, ly, tx, ty;
-  gpmem_t av;
+  pari_sp av;
   GEN p1,num,den;
 
 
@@ -1541,7 +1543,7 @@ gaffect(GEN x, GEN y)
 GEN
 co8(GEN x, long prec)
 {
-  gpmem_t av=avma, tetpil;
+  pari_sp av=avma, tetpil;
   GEN p1, p = (GEN) x[1];
 
   p1 = subii(sqri((GEN)p[3]), shifti((GEN)p[2],2));
@@ -1565,7 +1567,7 @@ cvtop(GEN x, GEN p, long l)
 {
   GEN p1,p2,p3;
   long n;
-  gpmem_t av, tetpil;
+  pari_sp av, tetpil;
 
   if (typ(p)!=t_INT)
     err(talker,"not an integer modulus in cvtop or gcvtop");
@@ -1654,7 +1656,7 @@ long
 gexpo(GEN x)
 {
   long tx=typ(x), lx, e, i, y;
-  gpmem_t av;
+  pari_sp av;
 
   switch(tx)
   {
@@ -1691,33 +1693,6 @@ sizedigit(GEN x)
   return gcmp0(x)? 0: (long) ((gexpo(x)+1) * L2SL10) + 1;
 }
 
-#if 0
-/* Normalize series x in place.
- * Assumption: x,x[2],...,x[lg(x)-1] have been created in that order.
- * All intermediate objects will be destroyed.
- */
-GEN
-normalize(GEN x)
-{
-  long i,j, lx = lg(x);
-
-  if (typ(x)!=t_SER) err(typeer,"normalize");
-  if (lx==2) { setsigne(x,0); avma = (gpmem_t) x; return x; }
-  if (! isexactzero((GEN)x[2])) { setsigne(x,1); return x; }
-
-  for (i=3; i<lx; i++)
-    if (! isexactzero((GEN)x[i]))
-    {
-      gpmem_t tetpil = avma;
-      GEN p1 = cgetg(lx-i+2,t_SER);
-      p1[1] = evalsigne(1) | evalvalp(valp(x)+i-2) | evalvarn(varn(x));
-      j=i; i=2; while (j<lx) p1[i++] = lcopy((GEN)x[j++]);
-      return gerepile((gpmem_t) (x+lx),tetpil,p1);
-    }
-  avma = (gpmem_t) (x+lx); return zeroser(varn(x),lx-2);
-}
-#else
-
 /* normalize series. avma is not updated */
 GEN
 normalize(GEN x)
@@ -1738,7 +1713,6 @@ normalize(GEN x)
     }
   return zeroser(varn(x),lx-2);
 }
-#endif
 
 GEN
 normalizepol_i(GEN x, long lx)
@@ -1789,66 +1763,66 @@ listcreate(long n)
   if (n<0) err(talker,"negative length in listcreate");
   n += 2;
   if (n & ~LGEFBITS) err(talker,"list too long (max = %ld)",LGEFBITS-2);
-  list=cgetg(n,t_LIST); list[1]=evallgef(2);
-  return list;
+  list = cgetg(n,t_LIST);
+  list[1] = evallgef(2); return list;
 }
 
 static void
 listaffect(GEN list, long index, GEN object)
 {
-  if (index<lgef(list) && isclone(list[index]))
-    gunclone((GEN)list[index]);
-  list[index]=lclone(object);
+  if (index < lgef(list) && isclone(list[index])) gunclone((GEN)list[index]);
+  list[index] = lclone(object);
 }
 
 void
 listkill(GEN list)
 {
-  long lx,i;
+  long l, i;
 
-  if (typ(list)!=t_LIST) err(typeer,"listkill");
-  lx=lgef(list);
-  for (i=2; i<lx; i++)
+  if (typ(list) != t_LIST) err(typeer,"listkill");
+  l = lgef(list);
+  for (i=2; i<l; i++)
     if (isclone(list[i])) gunclone((GEN)list[i]);
-  list[1]=evallgef(2); return;
+  list[1] = evallgef(2);
 }
 
 GEN
 listput(GEN list, GEN object, long index)
 {
-  long lx=lgef(list);
+  long l = lgef(list);
 
-  if (typ(list)!=t_LIST) err(typeer,"listput");
+  if (typ(list) != t_LIST) err(typeer,"listput");
   if (index < 0) err(talker,"negative index (%ld) in listput", index);
-  if (!index || index >= lx-1)
+  if (!index || index >= l-1)
   {
-    index = lx-1; lx++;
-    if (lx > lg(list))
+    index = l-1; l++;
+    if (l > lg(list))
       err(talker,"no more room in this list (size %ld)", lg(list)-2);
   }
-  listaffect(list,index+1,object);
-  list[1]=evallgef(lx);
+  listaffect(list, index+1, object);
+  list[1] = evallgef(l);
   return (GEN)list[index+1];
 }
 
 GEN
 listinsert(GEN list, GEN object, long index)
 {
-  long lx = lgef(list), i;
+  long l = lgef(list), i;
 
-  if (typ(list)!=t_LIST) err(typeer,"listinsert");
-  if (index <= 0 || index > lx-1) err(talker,"bad index in listinsert");
-  lx++; if (lx > lg(list)) err(talker,"no more room in this list");
+  if (typ(list) != t_LIST) err(typeer,"listinsert");
+  if (index <= 0 || index > l-1) err(talker,"bad index in listinsert");
+  l++; if (l > lg(list)) err(talker,"no more room in this list");
 
-  for (i=lx-2; i > index; i--) list[i+1]=list[i];
+  for (i=l-2; i > index; i--) list[i+1] = list[i];
   list[index+1] = lclone(object);
-  list[1] = evallgef(lx); return (GEN)list[index+1];
+  list[1] = evallgef(l);
+  return (GEN)list[index+1];
 }
 
 GEN
 gtolist(GEN x)
 {
-  long tx,lx,i;
+  long tx, lx, i;
   GEN list;
 
   if (!x) { list = cgetg(2, t_LIST); list[1] = evallgef(2); return list; }
@@ -1872,24 +1846,24 @@ gtolist(GEN x)
 GEN
 listconcat(GEN list1, GEN list2)
 {
-  long i,l1,lx;
+  long i, l1, lx;
   GEN list;
 
-  if (typ(list1)!=t_LIST || typ(list2)!=t_LIST)
-    err(typeer,"listconcat");
-  l1=lgef(list1)-2; lx=l1+lgef(list2);
+  if (typ(list1) != t_LIST || typ(list2) != t_LIST) err(typeer,"listconcat");
+  l1 = lgef(list1)-2;
+  lx = l1 + lgef(list2);
   list = listcreate(lx-2);
-  for (i=2; i<=l1+1; i++) listaffect(list,i,(GEN)list1[i]);
-  for (   ; i<lx; i++) listaffect(list,i,(GEN)list2[i-l1]);
-  list[1]=evallgef(lx); return list;
+  for (i=2; i<=l1+1; i++) listaffect(list, i, (GEN)list1[i]);
+  for (   ; i<lx;    i++) listaffect(list, i, (GEN)list2[i-l1]);
+  list[1] = evallgef(lx); return list;
 }
 
 GEN
 listsort(GEN list, long flag)
 {
-  long i, c=list[1], lx = lgef(list)-1;
-  gpmem_t av=avma;
-  GEN perm,vec,l;
+  long i, c = list[1], lx = lgef(list)-1;
+  pari_sp av = avma;
+  GEN perm, vec, l;
 
   if (typ(list) != t_LIST) err(typeer,"listsort");
   vec = list+1;
@@ -1905,10 +1879,10 @@ listsort(GEN list, long flag)
         vec[++c] = l[i];
       else
         if (isclone(l[i])) gunclone((GEN)l[i]);
-    setlgef(list,c+2);
+    setlgef(list, c+2);
   }
   else
     for (i=1; i<lx; i++) vec[i] = l[i];
 
-  avma=av; return list;
+  avma = av; return list;
 }
