@@ -1,4 +1,4 @@
-/* $Id: rootpol.c,v 1.59 2002/09/07 01:28:53 karim Exp $
+/* $Id: rootpol.c,v 1.65 2003/02/05 13:37:41 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -21,6 +21,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*******************************************************************/
 #include "pari.h"
 
+extern GEN TR_pol(GEN P, GEN c);
 extern GEN polrecip_i(GEN x);
 #define pariINFINITY 100000
 #define NEWTON_MAX 10
@@ -63,7 +64,7 @@ quickmulcc(GEN x, GEN y)
     }
     if (ty==t_COMPLEX)
     {
-      gpmem_t av, tetpil;
+      pari_sp av, tetpil;
       GEN p1,p2;
 
       z=cgetg(3,t_COMPLEX); av=avma;
@@ -96,7 +97,7 @@ mysquare(GEN p)
 {
   GEN s,aux1,aux2;
   long i, j, n=degpol(p), nn;
-  gpmem_t ltop, lbot;
+  pari_sp ltop, lbot;
 
   if (n==-1) return gcopy(p);
   nn=n<<1; s=cgetg(nn+3,t_POL);
@@ -141,7 +142,7 @@ karasquare(GEN p)
 {
   GEN p1,s0,s1,s2,aux;
   long n=degpol(p),n0,n1,i,var,nn0;
-  gpmem_t ltop;
+  pari_sp ltop;
 
   if (n<=KARASQUARE_LIMIT) return mysquare(p);
   ltop=avma;
@@ -173,7 +174,7 @@ cook_square(GEN p)
 {
   GEN p0,p1,p2,p3,q,aux0,aux1,r,aux,plus,moins;
   long n=degpol(p),n0,n3,i,j,var;
-  gpmem_t ltop = avma;
+  pari_sp ltop = avma;
 
   if (n<=COOK_SQUARE_LIMIT) return karasquare(p);
 
@@ -337,9 +338,10 @@ log2ir(GEN x)
   if (signe(x)==0) return (double) -pariINFINITY;
   if (typ(x)==t_INT)
   {
-    if (lgefint(x)==3) return (double) log2( (double)(ulong) x[2]);
-    l=(double)(ulong) x[2]+
-	(double)(ulong) x[3] / exp2((double) BITS_IN_LONG);
+    GEN m=int_MSW(x);
+    if (lgefint(x)==3) return (double) log2( (double)(ulong) *m);
+    l=(double)(ulong) *m +
+	(double)(ulong) *int_precW(m) / exp2((double) BITS_IN_LONG);
     return log2(l)+ (double) BITS_IN_LONG * (lgefint(x)-3.);
   }
   /* else x is real */
@@ -491,40 +493,24 @@ mygprec(GEN x, long bitprec)
   return y;
 }
 
-/* the round fonction has a bug in pari. Thus I create mygfloor, using gfloor
-which has no bug (destroy z)*/
+/* normalize a polynomial p, that is change it with coefficients in Z[i],
+after making product by 2^shift */
 static GEN
-mygfloor(GEN z)
+pol_to_gaussint(GEN p, long shift)
 {
-  if (typ(z)!=t_COMPLEX) return gfloor(z);
-  z[1]=lfloor((GEN)z[1]); z[2]=lfloor((GEN)z[2]); return z;
+  long i, l = lgef(p);
+  GEN q = cgetg(l, t_POL); q[1] = p[1];
+  for (i=2; i<l; i++) q[i] = (long)gfloor2n((GEN)p[i], shift);
+  return q;
 }
 
 /* returns a polynomial q in (Z[i])[x] keeping bitprec bits of p */
 static GEN
 eval_rel_pol(GEN p,long bitprec)
 {
-  long e=gexpo(p),n=lgef(p),i,shift;
-  GEN q = gprec(p,(long) ((double) bitprec * L2SL10)+2);
-
-  shift=bitprec-e+1;
-  for (i=2; i<n; i++)
-    q[i]=(long) mygfloor(myshiftic((GEN)q[i],shift));
-  return q;
+  return pol_to_gaussint(p, bitprec-gexpo(p)+1);
 }
 
-/* normalize a polynomial p, that is change it with coefficients in Z[i],
-after making product by 2^bitprec */
-static void
-pol_to_gaussint(GEN p, long bitprec)
-{
-  long i,n=lgef(p);
-  for (i=2; i<n; i++)
-  {
-    myshiftrc((GEN) p[i],bitprec);
-    p[i]=(long) mygfloor((GEN) p[i]);
-  }
-}
 
 /* returns p(R*x)/R^n (in R or R[i]), n=deg(p), bitprec bits of precision */
 static GEN
@@ -560,7 +546,7 @@ homothetie2n(GEN p, long e)
 static void
 homothetie_gauss(GEN p, long e,long f)
 {
-  if (e||f)
+  if (e || f)
   {
     long i, n=lgef(p)-1;
     for (i=2; i<=n; i++) p[i]=(long) myshiftic((GEN) p[i],f+(n-i)*e);
@@ -574,7 +560,7 @@ static double
 lower_bound(GEN p, long *k, double eps)
 {
   long n=degpol(p), i, j;
-  gpmem_t ltop=avma;
+  pari_sp ltop=avma;
   GEN a,s,S,icd;
   double r,*rho;
 
@@ -610,7 +596,7 @@ GEN
 max_modulus(GEN p, double tau)
 {
   GEN r,q,aux,gunr;
-  gpmem_t av, ltop = avma;
+  pari_sp av, ltop = avma;
   long i,k,n=degpol(p),nn,bitprec,M,e;
   double rho,eps, tau2 = (tau > 3.0)? 0.5: tau/6.;
 
@@ -625,8 +611,7 @@ max_modulus(GEN p, double tau)
   e = findpower(q);
   homothetie2n(q,e);
   affsi(e, r);
-  q = mygprec(q,bitprec+(n<<1));
-  pol_to_gaussint(q,bitprec);
+  q = pol_to_gaussint(q, bitprec);
   M = (long) (log2( log(4.*n) / (2*tau2) )) + 2;
   nn = n;
   for (i=0,e=0;;)
@@ -646,7 +631,7 @@ max_modulus(GEN p, double tau)
     eps = -1/log(tau2); /* > 0 */
     e = findpower(q);
   }
-  if (!signe(r)) { avma = ltop; return realun(DEFAULTPREC); } 
+  if (!signe(r)) { avma = ltop; return realun(DEFAULTPREC); }
   r = itor(r, DEFAULTPREC);
   setexpo(r, expo(r) - M);
   rho = rtodbl(r);
@@ -660,16 +645,15 @@ static GEN
 modulus(GEN p, long k, double tau)
 {
   GEN q,gunr;
-  long i, kk=k, imax, n=degpol(p), nn, bitprec, decprec, e;
-  gpmem_t av, ltop=avma;
+  long i, kk=k, imax, n=degpol(p), nn, bitprec, e;
+  pari_sp av, ltop=avma;
   double tau2,r;
 
   tau2 = tau/6;
   bitprec= (long) ((double) n*(2.+log2(3.*(double) n)+log2(1./tau2)));
-  decprec=(long) ((double) bitprec * L2SL10)+1;
   gunr=myrealun(bitprec);
   av = avma;
-  q = gprec(p,decprec);
+  q = gprec_w(p, 3 + (bitprec>>TWOPOTBITS_IN_LONG));
   q = gmul(gunr,q);
   e = newton_polygon(q,k);
   homothetie2n(q,e);
@@ -702,7 +686,7 @@ pre_modulus(GEN p, long k, double tau, GEN rmin, GEN rmax)
 {
   GEN R, q, aux;
   long n=degpol(p), i, imax, imax2, bitprec;
-  gpmem_t ltop=avma, av;
+  pari_sp ltop=avma, av;
   double tau2, aux2;
 
   tau2=tau/6.;
@@ -737,7 +721,7 @@ pre_modulus(GEN p, long k, double tau, GEN rmin, GEN rmax)
 static GEN
 min_modulus(GEN p, double tau)
 {
-  gpmem_t av=avma;
+  pari_sp av=avma;
   GEN r;
 
   if (isexactzero((GEN)p[2])) return realzero(3);
@@ -752,7 +736,7 @@ dual_modulus(GEN p, GEN R, double tau, long l)
 {
   GEN q;
   long i, imax, k, delta_k=0, n=degpol(p), nn, nnn, valuat, bitprec, ll=l;
-  gpmem_t ltop=avma;
+  pari_sp ltop=avma;
   double logmax,aux,tau2;
 
   tau2=7.*tau/8.;
@@ -813,7 +797,7 @@ gmulbyi(GEN z)
 static void
 fft(GEN Omega, GEN p, GEN f, long Step, long l)
 {
-  gpmem_t ltop;
+  pari_sp ltop;
   long i,l1,l2,l3,rap=Lmax/l,rapi,Step4;
   GEN f1,f2,f3,f02,f13,g02,g13,ff;
 
@@ -930,7 +914,7 @@ parameters(GEN p, double *mu, double *gamma,
 {
   GEN q,pc,Omega,coef,RU,prim,aux,aux0,ggamma,gx,mygpi;
   long n=degpol(p), bitprec, NN, K, i, j;
-  gpmem_t ltop2, ltop=avma, limite=stack_lim(ltop, 1);
+  pari_sp av2, av=avma, limite=stack_lim(av, 1);
   double lx;
 
   bitprec=gexpo(p)+(long)param2+8;
@@ -953,51 +937,47 @@ parameters(GEN p, double *mu, double *gamma,
   ggamma = gzero;
   aux0 = myrealun(bitprec);
   if (polreal) K=K/2+1;
-  ltop2=avma;
+  av2=avma;
   for (i=0; i<K; i++)
   {
     aux = aux0;
     for (j=0; j<=n; j++)
     {
-      pc[j]=lmul((GEN)q[j+2],aux);
-      aux=gmul(aux,RU); /* RU = prim^i, aux=prim^(ij) */
+      pc[j] = lmul((GEN)q[j+2],aux);
+      aux = gmul(aux,RU); /* RU = prim^i, aux=prim^(ij) */
     }
 
     fft(Omega,pc,coef,1,Lmax);
     for (j=0; j<Lmax; j++)
     {
-      aux=gprec((GEN)coef[j],DEFAULTPREC);
-      gx=gabs(aux,DEFAULTPREC);
-      lx=gtodouble(mplog(gx));
-      if (lx<*mu) *mu=lx;
+      aux = gprec_w((GEN)coef[j], DEFAULTPREC);
+      gx = gabs(aux, DEFAULTPREC);
+      lx = gtodouble(mplog(gx));
+      if (lx < *mu) *mu = lx;
       if (polreal && (i>0 && i<K-1))
-      {
-	gx=gdiv(gdeux,gx);
-	ggamma=gadd(ggamma,gx);
-      }
+	ggamma = gadd(ggamma, gdiv(gdeux,gx));
       else
-	ggamma=gadd(ggamma,ginv(gx));
+	ggamma = gadd(ggamma, ginv(gx));
     }
-    RU=gmul(RU,prim);
-    if (low_stack(limite, stack_lim(ltop,1)))
+    RU = gmul(RU, prim);
+    if (low_stack(limite, stack_lim(av,1)))
     {
-      GEN *gptr[2];
       if(DEBUGMEM>1) err(warnmem,"parameters");
-      gptr[0]=&ggamma; gptr[1]=&RU; gerepilemany(ltop2,gptr,2);
+      gerepileall(av2,2, &ggamma,&RU);
     }
   }
   ggamma=gdivgs(ggamma,NN);
-  *gamma=gtodouble(glog(ggamma,DEFAULTPREC))/log(2.);
-  avma=ltop;
+  *gamma=gtodouble(glog(ggamma,DEFAULTPREC)) / log(2.);
+  avma=av;
 }
 
 /* NN is a multiple of Lmax */
 static void
 dft(GEN p, long k, long NN, long bitprec, GEN F, GEN H, long polreal)
 {
-  GEN Omega,q,qd,pc,pdc,alpha,beta,gamma,RU,aux,U,W,mygpi,prim,prim2,*gptr[3];
+  GEN Omega,q,qd,pc,pdc,alpha,beta,gamma,RU,aux,U,W,mygpi,prim,prim2;
   long n=degpol(p),i,j,K;
-  gpmem_t ltop;
+  pari_sp ltop;
 
   mygpi=mppi(bitprec/BITS_IN_LONG+3);
   aux = gdivgs(mygpi,NN/2); /* 2 Pi/NN */
@@ -1072,8 +1052,7 @@ dft(GEN p, long k, long NN, long bitprec, GEN F, GEN H, long polreal)
       }
     }
     prim2=gmul(prim2,prim);
-    gptr[0]=&W; gptr[1]=&U; gptr[2]=&prim2;
-    gerepilemany(ltop,gptr,3);
+    gerepileall(ltop,3, &W,&U,&prim2);
   }
 
   for (i=1; i<=k; i++)
@@ -1094,7 +1073,7 @@ static GEN
 refine_H(GEN F, GEN G, GEN HH, long bitprec, long shiftbitprec)
 {
   GEN H=HH,D,aux;
-  gpmem_t ltop=avma, limite=stack_lim(ltop, 1);
+  pari_sp ltop=avma, limite=stack_lim(ltop, 1);
   long error=0,i,bitprec1,bitprec2;
 
   D=gsub(gun,gres(gmul(HH,G),F)); error=gexpo(D);
@@ -1104,9 +1083,8 @@ refine_H(GEN F, GEN G, GEN HH, long bitprec, long shiftbitprec)
   {
     if (low_stack(limite, stack_lim(ltop,1)))
     {
-      GEN *gptr[2]; gptr[0]=&D; gptr[1]=&H;
       if(DEBUGMEM>1) err(warnmem,"refine_H");
-      gerepilemany(ltop,gptr,2);
+      gerepileall(ltop,2, &D,&H);
     }
     bitprec1=-error+shiftbitprec;
     aux=gmul(mygprec(H,bitprec1),mygprec(D,bitprec1));
@@ -1130,7 +1108,7 @@ refine_F(GEN p, GEN *F, GEN *G, GEN H, long bitprec, double gamma)
   GEN pp,FF,GG,r,HH,f0;
   long error, i, bitprec1=0, bitprec2, shiftbitprec;
   long shiftbitprec2, n=degpol(p), enh, normF, normG;
-  gpmem_t ltop=avma, limite=stack_lim(ltop, 1);
+  pari_sp ltop=avma, limite=stack_lim(ltop, 1);
 
   FF=*F; HH=H;
   GG=poldivres(p,*F,&r);
@@ -1150,9 +1128,8 @@ refine_F(GEN p, GEN *F, GEN *G, GEN H, long bitprec, double gamma)
     }
     if (low_stack(limite, stack_lim(ltop,1)))
     {
-      GEN *gptr[4]; gptr[0]=&FF; gptr[1]=&GG; gptr[2]=&r; gptr[3]=&HH;
       if(DEBUGMEM>1) err(warnmem,"refine_F");
-      gerepilemany(ltop,gptr,4);
+      gerepileall(ltop,4, &FF,&GG,&r,&HH);
     }
 
     bitprec1=-error+shiftbitprec2;
@@ -1188,7 +1165,7 @@ split_fromU(GEN p, long k, double delta, long bitprec,
   GEN pp,FF,GG,H;
   long n=degpol(p),NN,bitprec2;
   int polreal=isreal(p);
-  gpmem_t ltop;
+  pari_sp ltop;
   double mu,gamma;
 
   pp=gdiv(p,(GEN)p[2+n]);
@@ -1257,39 +1234,13 @@ scalepol(GEN p, GEN R, long bitprec)
   return q;
 }
 
-extern GEN addshiftpol(GEN x, GEN y, long d);
-
-/* returns q(x) = p(x+b) */
-static GEN
-shiftpol(GEN p, GEN b)
-{
-  long i;
-  gpmem_t av = avma, limit = stack_lim(av, 1);
-  GEN q = gzero;
-
-  if (gcmp0(b)) return p;
-
-  for (i=lgef(p)-1; i>=2; i--)
-  {
-    if (!signe(q)) { q = scalarpol((GEN)p[i], varn(p)); continue; }
-    q = addshiftpol(q, gmul(b,q), 1); /* q = q*(x + b) */
-    q[2] = ladd((GEN)q[2], (GEN)p[i]); /* q = q + p[i] */
-    if (low_stack(limit, stack_lim(av,1)))
-    {
-      if(DEBUGMEM>1) err(warnmem,"rootpol.c:shiftpol()");
-      q = gerepilecopy(av, q);
-    }
-  }
-  return gerepilecopy(av, q);
-}
-
 /* return (conj(a)X-1)^n * p[ (X-a) / (conj(a)X-1) ] */
 static GEN
 conformal_pol(GEN p, GEN a, long bitprec)
 {
   GEN r,pui,num,aux, unr = myrealun(bitprec);
   long n=degpol(p), i;
-  gpmem_t av, limit;
+  pari_sp av, limit;
 
   aux = pui = cgetg(4,t_POL);
   pui[1] = evalsigne(1) | evalvarn(varn(p)) | evallgef(4);
@@ -1308,9 +1259,8 @@ conformal_pol(GEN p, GEN a, long bitprec)
     aux = gmul(pui,aux);
     if (low_stack(limit, stack_lim(av,2)))
     {
-      GEN *gptr[2]; gptr[0] = &r; gptr[1] = &aux;
       if(DEBUGMEM>1) err(warnmem,"rootpol.c:conformal_pol()");
-      gerepilemany(av, gptr, 2);
+      gerepileall(av,2, &r,&aux);
     }
   }
 }
@@ -1380,23 +1330,23 @@ static void
 conformal_mapping(GEN *radii, GEN ctr, GEN p, long k, long bitprec,
                   double aux, GEN *F,GEN *G)
 {
-  long bitprec2, n=degpol(p), decprec, i;
-  gpmem_t ltop = avma, av;
-  GEN q,FF,GG,a,R, *gptr[2];
+  long bitprec2, n=degpol(p), i;
+  pari_sp ltop = avma, av;
+  GEN q,FF,GG,a,R;
   GEN rho,invrho;
   double delta,param,param2;
 
   bitprec2=bitprec+(long) (n*(2.*log2(2.732)+log2(1.5)))+1;
-  a=gsqrt(stoi(3), 2*MEDDEFAULTPREC - 2);
-  a=gmul(mygprec(a,bitprec2),mygprec(ctr,bitprec2));
-  a=gdivgs(a,-6); /* a = -ctr/2sqrt(3) */
+  a = gsqrt(stoi(3), 2*MEDDEFAULTPREC - 2);
+  a = gmul(mygprec(a,bitprec2),mygprec(ctr,bitprec2));
+  a = gdivgs(a,-6); /* a = -ctr/2sqrt(3) */
 
   av = avma; q = mygprec(p,bitprec2);
   q = conformal_pol(q,a,bitprec2);
   for (i=1; i<=n; i++)
     if (signe(radii[i])) /* updating array radii */
     {
-      gpmem_t a = avma;
+      pari_sp a = avma;
       GEN p1 = gsqr(radii[i]);
       /* 2(r^2 - 1) / (r^2 - 3(r-1)) */
       p1 = divrr(gmul2n((subrs(p1,1)),1),
@@ -1411,8 +1361,7 @@ conformal_mapping(GEN *radii, GEN ctr, GEN p, long k, long bitprec,
   bitprec2 += (long) (((double)n) * fabs(log2ir(rho)) + 1.);
   R = mygprec(invrho,bitprec2);
   q = scalepol(q,R,bitprec2);
-  gptr[0] = &q; gptr[1] = &R;
-  gerepilemany(av,gptr,2);
+  gerepileall(av,2, &q,&R);
 
   optimize_split(q,k,delta,bitprec2,&FF,&GG,param,param2);
   bitprec2 += n; R = ginv(R);
@@ -1422,16 +1371,14 @@ conformal_mapping(GEN *radii, GEN ctr, GEN p, long k, long bitprec,
   a = mygprec(a,bitprec2);
   FF = conformal_pol(FF,a,bitprec2);
   GG = conformal_pol(GG,a,bitprec2);
-  a = ginv(gsub(gun, gnorm(a)));
-  a = glog(a,(long) (bitprec2 * L2SL10)+1);
 
-  decprec = (long) ((bitprec+n) * L2SL10)+1;
-  FF = gmul(FF,gexp(gmulgs(a,k),decprec));
-  GG = gmul(GG,gexp(gmulgs(a,n-k),decprec));
+  a = mplog( ginv(gsub(gun, gnorm(a))) );
+  FF = gmul(FF, mpexp(mulrs(a,k)));
+  GG = gmul(GG, mpexp(mulrs(a,n-k)));
 
   *F = mygprec(FF,bitprec+n);
   *G = mygprec(GG,bitprec+n);
-  gptr[0]=F; gptr[1]=G; gerepilemany(ltop,gptr,2);
+  gerepileall(ltop,2, F,G);
 }
 
 static GEN
@@ -1479,9 +1426,9 @@ split_2(GEN p, long bitprec, GEN ctr, double thickness, GEN *F, GEN *G)
     aux = rtodbl(mplog(divrr(rmax,rmin))) / (j-i) / 4.;
     k = dual_modulus(p,rho,aux, min(i,n+1-j));
     if (k-i < j-k-1 || (k-i == j-k-1 && 2*k > n))
-      { rmax=rho; j=k+1; affrr(mulrr(rho, dbltor(exp(-aux))), radii[j]); }
+      { rmax=rho; j=k+1; affrr(mulrr(rho, mpexp(dbltor(-aux))), radii[j]); }
     else
-      { rmin=rho; i=k; affrr(mulrr(rho, dbltor(exp(aux))), radii[i]); }
+      { rmin=rho; i=k; affrr(mulrr(rho, mpexp(dbltor(aux))), radii[i]); }
   }
   aux = rtodbl(mplog(divrr(rmax, rmin)));
 
@@ -1542,7 +1489,7 @@ split_1(GEN p, long bitprec, GEN *F, GEN *G)
   ctr = NULL; imax = polreal? 3: 4;
   for (i=1; i<=imax; i++)
   {
-    qq = shiftpol(q, (GEN)v[i]);
+    qq = TR_pol(q, (GEN)v[i]);
     rmin = min_modulus(qq,0.05);
     if (cmpsr(3, mulrr(rmin, thickness)) > 0)
     {
@@ -1556,8 +1503,8 @@ split_1(GEN p, long bitprec, GEN *F, GEN *G)
   bitprec2 = bitprec + gexpo(newq) - ep + (long)((double)n*log2(3.)+1);
   split_2(newq,bitprec2,ctr, rtodbl(mplog(thickness)),&FF,&GG);
   r = gneg(mygprec(ctr,bitprec2));
-  FF = shiftpol(FF,r);
-  GG = shiftpol(GG,r);
+  FF = TR_pol(FF,r);
+  GG = TR_pol(GG,r);
 
   gr = ginv(gr); bitprec2 = bitprec - ep + gexpo(FF)+gexpo(GG);
   *F = scalepol(FF,gr,bitprec2);
@@ -1581,7 +1528,7 @@ split_0_2(GEN p, long bitprec, GEN *F, GEN *G)
 
   q=mygprec(p,bitprec2);
   b=gdivgs(gdiv((GEN)q[n+1],(GEN)q[n+2]),-n);
-  q = shiftpol(q,b);
+  q = TR_pol(q,b);
 
   k=0; eq=gexpo(q);
   while
@@ -1605,8 +1552,8 @@ split_0_2(GEN p, long bitprec, GEN *F, GEN *G)
   }
   GG = mygprec(GG,bitprec2);
   b = mygprec(gneg(b),bitprec2);
-  *F = shiftpol(FF,b);
-  *G = shiftpol(GG,b); return 1;
+  *F = TR_pol(FF,b);
+  *G = TR_pol(GG,b); return 1;
 }
 
 /* put in F and G two polynomials such that |P-FG|<2^(-bitprec)|P|,
@@ -1780,8 +1727,8 @@ append_clone(GEN r, GEN a) { a = gclone(a); appendL(r, a); return a; }
 static GEN
 split_complete(GEN p, long bitprec, GEN roots_pol)
 {
-  long n=degpol(p), decprec;
-  gpmem_t ltop;
+  long n = degpol(p);
+  pari_sp ltop;
   GEN p1,F,G,a,b,m1,m2,m;
 
   if (n==1)
@@ -1792,9 +1739,8 @@ split_complete(GEN p, long bitprec, GEN roots_pol)
   ltop = avma;
   if (n==2)
   {
-    F=gsub(gsqr((GEN)p[3]),gmul2n(gmul((GEN)p[2],(GEN)p[4]),2));
-    decprec=(long) ((double) bitprec * L2SL10)+1;
-    F=gsqrt(F,decprec);
+    F = gsub(gsqr((GEN)p[3]), gmul2n(gmul((GEN)p[2],(GEN)p[4]), 2));
+    F = gsqrt(F, 3 + (bitprec >> TWOPOTBITS_IN_LONG));
     p1 = gmul2n((GEN)p[4],1);
     a = gneg_i(gdiv(gadd(F,(GEN)p[3]), p1));
     b =        gdiv(gsub(F,(GEN)p[3]), p1);
@@ -1901,7 +1847,7 @@ all_roots(GEN p, long bitprec)
 {
   GEN lc,pd,q,roots_pol,m;
   long bitprec0, bitprec2,n=degpol(p),i,e,h;
-  gpmem_t av;
+  pari_sp av;
 
   pd = poldeflate(p, &h); lc = leading_term(pd);
   e = 2*gexpo(cauchy_bound(pd)); if (e<0) e=0;
@@ -2051,7 +1997,7 @@ isrealappr(GEN x, long e)
 static int
 isconj(GEN x, GEN y, long e)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long i= (gexpo( gsub((GEN)x[1],(GEN)y[1]) ) < e
         && gexpo( gadd((GEN)x[2],(GEN)y[2]) ) < e);
   avma = av; return i;
@@ -2063,7 +2009,7 @@ isconj(GEN x, GEN y, long e)
 GEN
 roots(GEN p, long l)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long n,i,k,s,t,e;
   GEN c,L,p1,res,rea,com;
 
