@@ -1,4 +1,4 @@
-/* $Id: gp_init.c,v 1.17 2002/09/07 21:52:27 cvsadmin Exp $
+/* $Id: gp_init.c,v 1.19 2002/12/15 17:52:59 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -25,12 +25,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 extern void  allocatemem0(size_t newsize);
 extern GEN   default0(char *a, char *b, long flag);
-extern void  error0(GEN *g);
 extern GEN   extern0(char *cmd);
 extern void  gp_quit(void);
 extern GEN   input0(void);
 extern void  kill0(entree *ep);
-extern void  print0(GEN *g,long flag);
 extern GEN   read0(char *s);
 extern void  system0(char *cmd);
 extern GEN   trap0(char *e, char *f, char *r);
