@@ -1,4 +1,4 @@
-/* $Id: gp_rl.c,v 1.18 2002/09/05 11:38:39 karim Exp $
+/* $Id: gp_rl.c,v 1.29 2003/01/09 23:42:34 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -34,10 +34,14 @@ BEGINEXTERN
 #endif
 #ifdef READLINE_LIBRARY
 #  include <readline.h>
-#  include <history.h>
+#  ifdef HAS_HISTORY_H
+#    include <history.h>
+#  endif
 #else
 #  include <readline/readline.h>
-#  include <readline/history.h>
+#  ifdef HAS_HISTORY_H
+#    include <readline/history.h>
+#  endif
 #endif
 #ifndef HAS_RL_MESSAGE
 extern int rl_message (const char *, ...);
@@ -61,8 +65,6 @@ static entree *current_ep = NULL;
 
 static int pari_rl_back;
 extern RLCI rl_last_func;
-static int do_args_complete = 1;
-static int do_matched_insert = 0;
 static int did_init_matched = 0;
 
 #ifdef HAS_RL_SAVE_PROMPT
@@ -82,28 +84,40 @@ static int did_init_matched = 0;
 #  define COMPLETION_MATCHES ((CF)rl_completion_matches)
 #  define FILE_COMPLETION ((GF)rl_filename_completion_function)
 #  define USER_COMPLETION ((GF)rl_username_completion_function)
+#  define DING rl_ding
 #else
 #  define COMPLETION_MATCHES ((CF)completion_matches)
 #  define FILE_COMPLETION ((GF)filename_completion_function)
 #  define USER_COMPLETION ((GF)username_completion_function)
+#  define DING ding
 #endif
 
-#define ELECTRIC_PAREN 1
-#define ARGS_COMPLETE  2
 static int
-change_state(char *msg, int *opt, int count)
+change_state(char *msg, ulong flag, int count)
 {
-  int c;
+  int c = (readline_state & flag) != 0;
+  ulong o_readline_state = readline_state;
 
   switch(count)
   {
     default: c = 0; break; /* off */
     case -1: c = 1; break; /* on  */
-    case -2: c = 1 - *opt; /* toggle */
+    case -2: c = 1 - c; /* toggle */
+  }
+  if (c)
+    readline_state |= flag;
+  else {
+    readline_state &= ~flag;
+    if (!readline_state && o_readline_state)
+	readline_state = 1;
   }
-  *opt = c;
   SAVE_PROMPT();
-  rl_message("%s: %s.", msg, c? "on": "off");
+#if 0				/* Does not work well... */
+  rl_message("%s[%s: %s] %s", term_get_color(c_PROMPT),
+	     msg, c? "on": "off", term_get_color(c_INPUT));
+#else
+  rl_message("[%s: %s] ", msg, c? "on": "off");
+#endif
   c = rl_read_key();
   RESTORE_PROMPT();
   rl_clear_message();
@@ -118,7 +132,7 @@ pari_rl_complete(int count, int key)
 
   pari_rl_back = 0;
   if (count <= 0)
-    return change_state("complete args", &do_args_complete, count);
+    return change_state("complete args", DO_ARGS_COMPLETE, count);
 
   rl_begin_undo_group();
   if (rl_last_func == pari_rl_complete)
@@ -129,6 +143,30 @@ pari_rl_complete(int count, int key)
   rl_end_undo_group(); return ret;
 }
 
+static int did_matched_insert;
+
+static int
+pari_rl_matched_insert_suspend(int count, int key)
+{
+  ulong o_readline_state = readline_state;
+  (void)count; (void)key;
+
+  did_matched_insert = (readline_state & DO_MATCHED_INSERT);
+  readline_state &= ~DO_MATCHED_INSERT;
+  if (!readline_state && o_readline_state)
+    readline_state = 1;
+  return 1;
+}
+
+static int
+pari_rl_matched_insert_restore(int count, int key)
+{
+  (void)count; (void)key;
+  if (did_matched_insert)
+    readline_state |= DO_MATCHED_INSERT;
+  return 1;
+}
+
 static const char paropen[] = "([{";
 static const char parclose[] = ")]}";
 
@@ -139,9 +177,9 @@ pari_rl_matched_insert(int count, int key)
   int i = 0, ret;
 
   if (count <= 0)
-    return change_state("electric parens", &do_matched_insert, count);
+    return change_state("electric parens", DO_MATCHED_INSERT, count);
   while (paropen[i] && paropen[i] != key) i++;
-  if (!paropen[i] || !do_matched_insert || GP_DATA->flags & EMACS)
+  if (!paropen[i] || !(readline_state & DO_MATCHED_INSERT) || GP_DATA->flags & EMACS)
     return ((RLCI)rl_insert)(count,key);
   rl_begin_undo_group();
   ((RLCI)rl_insert)(count,key);
@@ -155,7 +193,7 @@ pari_rl_default_matched_insert(int count, int key)
 {
     if (!did_init_matched) {
 	did_init_matched = 1;
-	do_matched_insert = 1;
+	readline_state |= DO_MATCHED_INSERT;
     }
     return pari_rl_matched_insert(count, key);
 }
@@ -222,7 +260,7 @@ pari_rl_forward_sexp(int count, int key)
     }
     else
     {
-      fail: ding(); return 1;
+      fail: DING(); return 1;
     }
   }
   if (dir != 1 && move_point) rl_point++;
@@ -537,17 +575,39 @@ pari_completion(char *text, int START, int END)
     while (j <= END && isspace((int)rl_line_buffer[j])) j++;
     k = END;
     while (k > j && isspace((int)rl_line_buffer[k])) k--;
-    /* If we are in empty parens, output function help */
-    if (do_args_complete && k == j
+    /* If we are in empty parens, insert the default arguments */
+    if ((readline_state & DO_ARGS_COMPLETE) && k == j
          && (rl_line_buffer[j] == ')' || !rl_line_buffer[j])
 	 && (iend - i < MAX_KEYWORD)
 	 && ( strncpy(buf, rl_line_buffer + i, iend - i),
 	      buf[iend - i] = 0, 1)
 	 && (ep = is_entry(buf)) && ep->help)
      {
+#if 1
+      char *s = ep->help;
+
+      while (is_keyword_char(*s)) s++;
+      if (*s++ == '(')
+      { /* Function, print arguments! */
+        char *endh = s;
+        while (*endh && *endh != ')' && *endh != '(') endh++;
+        if (*endh == ')')
+        { /* Well-formed help.  */
+          char *str = strncpy((char*) gpmalloc(endh-s + 1), s, endh-s);
+          char **ret = (char**)gpmalloc(sizeof(char*)*2);
+          str[endh-s] = 0;
+          ret[0] = str; ret[1] = NULL;
+          if (GP_DATA->flags & EMACS) ret = matches_for_emacs("",ret);
+          return ret;
+        }
+      }
+#else			/* Why duplicate F1 (and emit a bell)?! */
       rl_print_aide(buf,h_RL);
+#ifdef HAS_RL_ATTEMPTED_COMPLETION_OVER
       rl_attempted_completion_over = 1;
+#endif
       return NULL;
+#endif
     }
   }
   for(i=END-1;i>=start;i--)
@@ -568,6 +628,7 @@ rl_short_help(int count, int key)
   int flag = (count < 0 || rl_last_func == rl_short_help)? h_RL|h_LONG: h_RL;
   int off = rl_point;
 
+  (void)key;
   /* func() with cursor on ')' following completion */
   if (off && rl_line_buffer[off-1] == '('
           && !is_keyword_char(rl_line_buffer[off])) off--;
@@ -620,10 +681,16 @@ init_readline(void)
   rl_attempted_completion_function = (rl_completion_func_t*) pari_completion;
 
   /* we always want the whole list of completions under emacs */
+#ifdef HAS_RL_COMPLETION_QUERY_ITEMS
   if (GP_DATA->flags & EMACS) rl_completion_query_items = 0x8fff;
-
-#define Bind(a,b,c) (((void(*)(int,Function*,Keymap)) rl_bind_key_in_map)\
+#endif
+#ifdef HAS_RL_BIND_KEY_IN_MAP
+#  define Bind(a,b,c) (((void(*)(int,Function*,Keymap)) rl_bind_key_in_map)\
   ((a), (Function*)(b), (c)))
+#else
+#  define Bind(a,b,c)
+#endif
+
 #define Defun(a,b,c) (((void(*)(const char*,Function*,int)) rl_add_defun)\
   ((a), (Function*)(b), (c)))
 
@@ -631,6 +698,8 @@ init_readline(void)
   Defun("long-help", rl_long_help, -1);
   Defun("pari-complete", pari_rl_complete, '\t');
   Defun("pari-matched-insert", pari_rl_default_matched_insert, -1);
+  Defun("pari-matched-insert-suspend", pari_rl_matched_insert_suspend, -1);
+  Defun("pari-matched-insert-restore", pari_rl_matched_insert_restore, -1);
   Defun("pari-forward-sexp", pari_rl_forward_sexp, -1);
   Defun("pari-backward-sexp", pari_rl_backward_sexp, -1);
 
@@ -645,6 +714,13 @@ init_readline(void)
   KSbind("[11~", rl_short_help,  emacs_meta_keymap); /* f1, xterm */
   KSbind("OP",   rl_short_help,  vi_movement_keymap); /* f1, vt100 */
   KSbind("[11~", rl_short_help,  vi_movement_keymap); /* f1, xterm */
+  /* XTerm may signal start/end of paste by eming F200/F201 */
+  /* XXXX For vi mode something more intelligent is needed - to switch to the
+     insert mode - and back when restoring. */
+  KSbind("[200~", pari_rl_matched_insert_suspend,  emacs_meta_keymap);  /* pre-paste xterm */
+  KSbind("[200~", pari_rl_matched_insert_suspend,  vi_movement_keymap); /* pre-paste xterm */
+  KSbind("[201~", pari_rl_matched_insert_restore,  emacs_meta_keymap);  /* post-paste xterm */
+  KSbind("[201~", pari_rl_matched_insert_restore,  vi_movement_keymap); /* post-paste xterm */
 #  endif
   Bind('(', pari_rl_matched_insert, emacs_standard_keymap);
   Bind('[', pari_rl_matched_insert, emacs_standard_keymap);
@@ -659,6 +735,98 @@ init_readline(void)
 #endif
 }
 
+static void
+print_escape_string(char *s)
+{
+  long l = strlen(s);
+  char *t, *t0 = gpmalloc(l * 3 + 3);
+
+  t = t0; *t++ = '"';
+  for ( ;*s; *t++ = *s++)
+    switch(*s)
+    {
+      case DATA_BEGIN:
+      case DATA_END:
+      case DATA_ESCAPE: *t++ = DATA_ESCAPE; continue;
+
+      case '\\':
+      case '"': *t++ = '\\'; continue;
+    }
+  *t++ = '"';
+  *t++ = 0; printf(t0); free(t0);
+}
+
+static char *
+completion_word(long end)
+{
+  char *s = rl_line_buffer + end, *found_quote = NULL;
+  long i;
+  /* truncate at cursor position */
+  *s = 0;
+  /* first look for unclosed string */
+  for (i=0; i < end; i++)
+  {
+    switch(rl_line_buffer[i])
+    {
+      case '"':
+        found_quote = found_quote? NULL: rl_line_buffer + i;
+        break;
+
+      case '\\': i++; break;
+    }
+
+  }
+  if (found_quote) return found_quote + 1; /* return next char after quote */
+
+  /* else find beginning of word */
+  while (s >  rl_line_buffer)
+  {
+    s--;
+    if (!is_keyword_char(*s)) { s++; break; }
+  }
+  return s;
+}
+
+/* completion required with cursor on s + pos. Complete wrt strict left
+ * prefix */
+void
+texmacs_completion(char *s, long pos)
+{
+  char **matches, *text;
+
+  if (rl_line_buffer) free(rl_line_buffer);
+  rl_line_buffer = pari_strdup(s);
+  text = completion_word(pos);
+  /* text = start of expression we complete */
+  rl_end = strlen(s)-1;
+  rl_point = pos;
+  matches = pari_completion(text, text - rl_line_buffer, pos);
+  printf("%cscheme:(tuple",DATA_BEGIN);
+  if (matches)
+  {
+    long i, prelen = (rl_line_buffer+pos) - text;
+    char *t = gpmalloc(prelen+1);
+    strncpy(t, text, prelen); t[prelen] = 0; /* prefix */
+    printf(" ");
+    print_escape_string(t); free(t);
+    for (i = matches[1]? 1: 0; matches[i]; i++)
+    {
+      printf(" ");
+      print_escape_string(matches[i] + prelen);
+      free(matches[i]);
+    }
+    free(matches);
+  }
+  printf(")%c", DATA_END);
+  fflush(stdout);
+}
+
+#ifndef HAS_HISTORY_H
+extern int history_length;
+typedef struct HIST_ENTRY__ {char *line; void *data;} HIST_ENTRY;
+extern HIST_ENTRY *history_get(int);
+#endif
+
 static int
 history_is_new(char *s)
 {
@@ -680,22 +848,29 @@ extern int input_loop(filtre_t *F, input_method *IM);
 static char *
 gprl_input(Buffer *b, char **endp, input_method *IM)
 {
-  long used = *endp - b->buf;
-  long left = b->len - used, l;
-  char *s;
-  
-  if (! (s = readline(IM->prompt)) ) return NULL; /* EOF */
-  l = strlen(s);
-  if ((ulong)left < l)
-  {
-    long incr = b->len;
+  ulong used = *endp - b->buf;
+  ulong left = b->len - used, l;
+  char *s, *t;
 
+  if (! (s = readline(IM->prompt)) ) return NULL; /* EOF */
+  gp_add_history(s); /* Makes a copy */
+  l = strlen(s) + 1;
+  /* put back \n that readline stripped. This is needed for
+   * { print("a
+   *   b"); }
+   * and conforms with the other input methods anyway. */
+  t = gpmalloc(l + 1);
+  strncpy(t, s, l-1);
+  t[l-1] = '\n';
+  t[l]   = 0; /* equivalent to sprintf(t,"%s\n", s) */
+  if (left < l)
+  {
+    ulong incr = b->len;
     if (incr < l) incr = l;
     fix_buffer(b, b->len + incr);
     *endp = b->buf + used;
   }
-  gp_add_history(s); /* Makes a copy */
-  return s;
+  return t;
 }
 
 static void
@@ -712,12 +887,12 @@ unblock_SIGINT(void)
  * Return 0: EOF
  *        1: got one line from readline or infile */
 int
-get_line_from_readline(char *prompt, filtre_t *F)
+get_line_from_readline(char *prompt, char *bare_prompt, filtre_t *F)
 {
   const int index = history_length;
   char *s;
   input_method IM;
- 
+
   IM.prompt = prompt;
   IM.getline= &gprl_input;
   IM.free = 1;
@@ -735,9 +910,9 @@ get_line_from_readline(char *prompt, filtre_t *F)
       }
       gp_add_history(s);
     }
-  
+
     /* update logfile */
-    if (logfile) fprintf(logfile, "%s%s\n",prompt,s);
+    if (logfile) fprintf(logfile, "%s%s\n",bare_prompt,s);
   }
   unblock_SIGINT(); /* bug in readline 2.0: need to unblock ^C */
   return 1;
