@@ -1,4 +1,4 @@
-/* $Id: Gnuplot.h,v 1.5 2001/03/13 17:56:52 karim Exp $
+/* $Id: Gnuplot.h,v 1.12 2003/01/15 20:45:36 cvsadmin Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -69,7 +69,7 @@ FILE *outfile = NULL;
 extern  FILE *gpoutfile;
 FILE *gpoutfile = NULL;
 
-static outfile_set;
+static int outfile_set;
 static void
 set_gpoutfile(void)
 {
@@ -199,7 +199,8 @@ TBOOLEAN screen_ok;
 
 void map3d_xy (double x, double y, double z, unsigned int *xt, unsigned int *yt)
 {
-    croak("Unsupported function map3d_xy called");
+  (void)x; (void)y; (void)z; (void)xt; (void)yt;
+  croak("Unsupported function map3d_xy called");
 }
 
 /* End of 3.7.0-devel additions */
@@ -209,10 +210,10 @@ void map3d_xy (double x, double y, double z, unsigned int *xt, unsigned int *yt)
 struct value*
 const_express(struct value*v)
 {
-    if (token[c_token].is_token)
-	croak("Expect a number, got a string");
-    *v = token[c_token++].l_val;
-    return v;
+  if (token[c_token].is_token)
+      croak("Expect a number, got a string");
+  *v = token[c_token++].l_val;
+  return v;
 }
 
 void*
@@ -431,18 +432,21 @@ myterm_table_not_loaded_v(void)
 static void
 myterm_table_not_loaded(char *s)
 {
+  (void)s;
   myterm_table_not_loaded_v();
 }
 
 static void
 myterm_table_not_loaded_vdd(double x, double y)
 {
+  (void)x; (void)y;
   myterm_table_not_loaded_v();
 }
 
 static double
 myterm_table_not_loaded_di(int flag)
 {
+  (void)flag;
   myterm_table_not_loaded_v();
   return 0;			/* NOT REACHED */
 }
@@ -454,11 +458,13 @@ myterm_table_not_loaded_u()
   return 0;
 }
 
-void myterm_table_not_loaded_v4i4d(int term_xmin, int term_xmax, 
+static void myterm_table_not_loaded_v4i4d(int term_xmin, int term_xmax, 
 			     int term_ymin, int term_ymax,
 			     double plot_xmin, double plot_xmax,
 			     double plot_ymin, double plot_ymax)
 {
+  (void)term_xmin; (void)term_xmax; (void)term_ymin; (void)term_ymax;
+  (void)plot_xmin; (void)plot_xmax; (void)plot_ymin; (void)plot_ymax;
   myterm_table_not_loaded_v();
 }
 
@@ -472,10 +478,18 @@ void myterm_table_not_loaded_v4i4d(int term_xmin, int term_xmax,
 #  define list_terms		(*my_term_ftablep->term_funcs[TTABLE_LIST])
 #  define plotsizes_scale	(*my_term_ftablep->set_sizesp)
 #  define plotsizes_scale_get	(*my_term_ftablep->get_sizesp)
-#  define set_mouse_feedback_rectangle	(*my_term_ftablep->mouse_feedback_func)
 
-#  define scaled_xmax()	((int)termprop(xmax)*plotsizes_scale_get(0))
-#  define scaled_ymax()	((int)termprop(ymax)*plotsizes_scale_get(1))
+#ifdef USE_SET_FEEDBACK_RECTANGLE
+/* If DLL has it, but was compiled with older Gnuplot.h */
+#  define set_mouse_feedback_rectangle	(*my_term_ftablep->mouse_feedback_func)
+#else
+#  define set_mouse_feedback_rectangle(term_xmin, term_xmax, term_ymin, term_ymax, plot_xmin, plot_xmax, plot_ymin, plot_ymax)	\
+	((my_term_ftablep->loaded & 2) ?	\
+	 ((*my_term_ftablep->mouse_feedback_func)(term_xmin, term_xmax, term_ymin, term_ymax, plot_xmin, plot_xmax, plot_ymin, plot_ymax), 0) : 0)
+#endif	/* defined USE_SET_FEEDBACK_RECTANGLE */
+
+#  define scaled_xmax()	((int)(termprop(xmax)*plotsizes_scale_get(0)))
+#  define scaled_ymax()	((int)(termprop(ymax)*plotsizes_scale_get(1)))
 
 #define USE_FUNCTION_FROM_TABLE
 
@@ -485,7 +499,7 @@ my_change_term(char*s,int l)
     SET_OUTFILE;
     if (!my_term_ftablep->change_term_p)
 	UNKNOWN_null();
-    return term = (struct termentry *)(*my_term_ftablep->change_term_p)(s,l);
+    return term = (*((struct termentry *(*)(char *s,int l))(my_term_ftablep->change_term_p)))(s,l);
 }
 
 #if 0
@@ -520,6 +534,7 @@ set_term_funcp2(FUNC_PTR change_p, TSET_FP tchange)
 void
 set_term_funcp3(FUNC_PTR change_p, void *term_p, TSET_FP tchange)
 {
+  (void)term_p;
   set_term_funcp2(change_p, tchange);
 }
 
@@ -533,6 +548,7 @@ set_term_ftable(struct t_ftable *p)
 extern struct t_ftable *get_term_ftable();
 
 #else /* !DYNAMIC_PLOTTING */
+#define set_mouse_feedback_rectangle  mys_mouse_feedback_rectangle
 
 extern struct termentry term_tbl[];
 extern double min_array[], max_array[];
@@ -571,7 +587,8 @@ plotsizes_get(int flag)	{ return (flag ? ysize : xsize); }
 
 struct t_ftable my_term_ftable =
 {
-	1, (FUNC_PTR)&change_term, &term_set_output,
+	2,		/* bit 2 means it has mys_mouse_feedback_rectangle */
+	(FUNC_PTR)&change_term, &term_set_output,
 	&plotsizes_scale, &plotsizes_get,
 	{&term_start_plot, &term_end_plot, 
 	 &term_start_multiplot, &term_end_multiplot, &term_init, &list_terms},
