@@ -1,4 +1,4 @@
-/* $Id: anal.c,v 1.114 2002/09/11 21:43:01 karim Exp $
+/* $Id: anal.c,v 1.133 2003/02/05 17:52:01 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -47,14 +47,19 @@ static void   skipidentifier();
 static void   skipseq();
 static void   skipstring();
 static void   skiptruc();
-static GEN    strtoGENstr_t();
 static entree *entry();
-static entree *installep(void *f,char *name,int l,int v,int add,entree **table);
 static entree *skipentry(void);
 
+static entree *installep(void *f,char *name,int l,int v,int add,entree **table);
+#define VAR_POLS_LONGS		7	/* 4 words for polx, 3 for polun */
+/* Is the name proper??? */
+#define SIZEOF_VAR_POLS		(VAR_POLS_LONGS*sizeof(long))
+
+
 extern void killbloc0(GEN x, int inspect);
 extern int term_width(void);
 extern GEN addsmulsi(long a, long b, GEN Y);
+extern GEN rpowsi(ulong a, GEN n, long prec);
 
 /* last time we began parsing an object of specified type */
 static struct
@@ -111,15 +116,15 @@ enum { PARSEMNU_TEMPL_TERM_NL, PARSEMNU_ARG_WHITESP };
     } else err(talker,reason,s); } STMT_END
 
 unsigned long
-parse_option_string(char *arg, char *template, long flag, char **failure, char **failure_arg)
+parse_option_string(char *arg, char *tmplate, long flag, char **failure, char **failure_arg)
 {
     unsigned long retval = 0;
-    char *etemplate = NULL;
+    char *etmplate = NULL;
 
     if (flag & PARSEMNU_TEMPL_TERM_NL)
-	etemplate = strchr(template, '\n');
-    if (!etemplate)
-	etemplate = template + strlen(template);
+	etmplate = strchr(tmplate, '\n');
+    if (!etmplate)
+	etmplate = tmplate + strlen(tmplate);
 
     if (failure)
 	*failure = NULL;
@@ -128,7 +133,7 @@ parse_option_string(char *arg, char *template, long flag, char **failure, char *
 	char *e, *id;
 	char *negated;			/* action found with 'no'-ID */
 	int negate;			/* Arg has 'no' prefix removed */
-	int l, action = 0, first = 1, singleton = 0;
+	unsigned int l, action = 0, first = 1, singleton = 0;
 	char b[80], *buf, *inibuf;
 
 	if (flag & PARSEMNU_ARG_WHITESP)
@@ -136,8 +141,7 @@ parse_option_string(char *arg, char *template, long flag, char **failure, char *
 	if (!*arg)
 	    break;
 	e = arg;
-	while (IS_ID(*e))
-	    e++;
+	while (IS_ID(*e)) e++;
 	/* Now the ID is whatever is between arg and e. */
 	l = e - arg;
 	if (l >= sizeof(b))
@@ -155,35 +159,35 @@ parse_option_string(char *arg, char *template, long flag, char **failure, char *
 	negate = 0;
 	negated = NULL;
       find:
-	id = template;
-	while ((id = strstr(id, buf)) && id < etemplate) {
+	id = tmplate;
+	while ((id = strstr(id, buf)) && id < etmplate) {
 	    if (IS_ID(id[l])) {		/* We do not allow abbreviations yet */
-		id = id + l;		/* False positive */
+		id += l;		/* False positive */
 		continue;
 	    }
-	    if ((id >= template + 2) && (IS_ID(id[-1]))) {
+	    if ((id >= tmplate + 2) && (IS_ID(id[-1]))) {
 		char *s = id;
 
-		if ( !negate && s >= template+3
+		if ( !negate && s >= tmplate+3
 		     && ((id[-1] == '_') || (id[-1] == '-')) )
 		    s--;
 		/* Check whether we are preceeded by "no" */
 		if ( negate		/* buf initially started with "no" */
-		     || (s < template+2) || (s[-1] != 'o') || (s[-2] != 'n')
-		     || (s >= template+3 && IS_ID(s[-3]))) {
-		    id = id + l;		/* False positive */
+		     || (s < tmplate+2) || (s[-1] != 'o') || (s[-2] != 'n')
+		     || (s >= tmplate+3 && IS_ID(s[-3]))) {
+		    id += l;		/* False positive */
 		    continue;
 		}
 		/* Found noID in the template! */
-		negated = id + l;
-		id = id + l;
+		id += l;
+		negated = id;
 		continue;		/* Try to find without 'no'. */
 	    }
 	    /* Found as is */
-	    id = id + l;
+	    id += l;
 	    break;
 	}
-	if ( !id && !negated && !negate 
+	if ( !id && !negated && !negate
 	     && (l > 2) && buf[0] == 'n' && buf[1] == 'o' ) {
 	    /* Try to find the flag without the prefix "no". */
 	    buf += 2; l -= 2;
@@ -362,7 +366,7 @@ set_analyseur(char *s)
 static GEN
 lisseq0(char *t, GEN (*f)(void))
 {
-  const gpmem_t av = avma;
+  const pari_sp av = avma;
   char *olds = analyseur, *olde = mark.start;
   GEN res;
 
@@ -415,13 +419,62 @@ readseq(char *c, int strict)
   return z;
 }
 
+static void
+check_proto(char *code)
+{
+  char *s = code, *old;
+  if (*s == 'l' || *s == 'v') s++;
+  while (*s && *s != '\n') switch (*s++)
+  {
+    case '&':
+    case '=':
+    case 'E':
+    case 'G':
+    case 'I':
+    case 'L':
+    case 'M':
+    case 'P':
+    case 'S':
+    case 'V':
+    case 'f':
+    case 'n':
+    case 'p':
+    case 'r':
+    case 'x': break;
+    case 's':
+      if (*s == '*') s++;
+      break;
+    case 'D':
+      if (*s == 'G' || *s == '&' || *s == 'n' || *s == 'I' || *s == 'V')
+      {
+        s++; break;
+      }
+      old = s; while (*s != ',') s++;
+      if (*s != ',') err(talker2, "missing comma", old, code);
+      break;
+    case ',': break;
+    case '\n': return; /* Before the mnemonic */
+
+    case 'l':
+    case 'v': err(talker2, "this code has to come first", s-1, code);
+    default: err(talker2, "unknown parser code", s-1, code);
+  }
+}
+
 entree *
 install(void *f, char *name, char *code)
 {
   long hash;
   entree *ep = is_entry_intern(name, functions_hash, &hash);
 
-  if (ep) err(warner,"[install] '%s' already there. Not replaced", name);
+  check_proto(code);
+  if (ep)
+  {
+    if (ep->valence != EpINSTALL)
+      err(talker,"[install] identifier '%s' already in use", name);
+    err(warner, "[install] updating '%s' prototype; module not reloaded", name);
+    if (ep->code) free(ep->code);
+  }
   else
   {
     char *s = name;
@@ -429,8 +482,8 @@ install(void *f, char *name, char *code)
       while (is_keyword_char(*++s)) /* empty */;
     if (*s) err(talker2,"not a valid identifier", s, name);
     ep = installep(f, name, strlen(name), EpINSTALL, 0, functions_hash + hash);
-    ep->code = pari_strdup(code);
   }
+  ep->code = pari_strdup(code);
   return ep;
 }
 
@@ -574,7 +627,7 @@ var_make_safe()
       if (EpVALENCE(ep) == EpVAR)
       { /* make sure ep->value is a COPY */
         var_cell *v = (var_cell*)ep->args;
-        if (v->flag == PUSH_VAL) changevalue(ep, (GEN)ep->value);
+        if (v && v->flag == PUSH_VAL) changevalue(ep, (GEN)ep->value);
       }
 }
 
@@ -641,7 +694,7 @@ kill0(entree *ep)
 static GEN
 seq(void)
 {
-  const gpmem_t av = avma, lim = stack_lim(av,1);
+  const pari_sp av = avma, lim = stack_lim(av,1);
   GEN res = gnil;
 
   for(;;)
@@ -671,7 +724,7 @@ gshift_r(GEN x, GEN n) { return gshift(x,-itos(n)); }
 static GEN
 expr(void)
 {
-  gpmem_t av = avma, lim = stack_lim(av, 2);
+  pari_sp av = avma, lim = stack_lim(av, 2);
   GEN aux,e,e1,e2,e3;
   F2GEN F1,F2,F3;
   int F0 = 0;
@@ -813,7 +866,7 @@ err_match(char *s, char c)
 static long
 readlong()
 {
-  const gpmem_t av = avma;
+  const pari_sp av = avma;
   const char *old = analyseur;
   long m;
   GEN arg = expr();
@@ -867,6 +920,164 @@ do_switch(int noparen, int matchcomma)
   return 0;
 }
 
+/********************************************************************/
+/**                                                                **/
+/**                            STRINGS                             **/
+/**                                                                **/
+/********************************************************************/
+
+static char*
+init_buf(long len, char **ptbuf, char **ptlim)
+{
+  char *buf = (char *)new_chunk(2 + len / sizeof(long));
+  *ptbuf = buf; *ptlim = buf + len; return buf;
+}
+
+static char*
+realloc_buf(char *bp, long len, char **ptbuf,char **ptlimit)
+{
+  char *buf = *ptbuf;
+  long newlen = ((*ptlimit - buf) + len) << 1;
+  long oldlen = bp - buf;
+
+  (void)init_buf(newlen, ptbuf, ptlimit);
+  memcpy(*ptbuf, buf, oldlen);
+  return *ptbuf + oldlen;
+}
+
+static char *
+expand_string(char *bp, char **ptbuf, char **ptlimit)
+{
+  char *tmp = NULL; /* -Wall */
+  long len = 0; /* -Wall */
+  int alloc = 1;
+
+  if (is_keyword_char(*analyseur))
+  {
+    char *s = analyseur;
+    do s++; while (is_keyword_char(*s));
+
+    if ((*s == '"' || *s == ',' || *s == ')') && !is_entry(analyseur))
+    { /* Do not create new user variable. Consider as a literal */
+      tmp = analyseur;
+      len = s - analyseur;
+      analyseur = s;
+      alloc = 0;
+    }
+  }
+
+  if (alloc)
+  {
+    pari_sp av = avma;
+    GEN p1 = expr();
+    if (br_status) err(breaker,"here (expanding string)");
+    tmp = GENtostr0(p1, &DFLT_OUTPUT, &gen_output);
+    len = strlen(tmp); avma = av;
+  }
+  if (ptlimit && bp + len > *ptlimit)
+    bp = realloc_buf(bp, len, ptbuf,ptlimit);
+  memcpy(bp,tmp,len); /* ignore trailing \0 */
+  if (alloc) free(tmp);
+  return bp + len;
+}
+
+static char *
+translate(char **src, char *s, char **ptbuf, char **ptlim)
+{
+  char *t = *src;
+  while (*t)
+  {
+    while (*t == '\\')
+    {
+      switch(*++t)
+      {
+	case 'e':  *s='\033'; break; /* escape */
+	case 'n':  *s='\n'; break;
+	case 't':  *s='\t'; break;
+	default:   *s=*t; if (!*t) err(talker,"unfinished string");
+      }
+      t++; s++;
+    }
+    if (*t == '"')
+    {
+      if (t[1] != '"') break;
+      t += 2; continue;
+    }
+    if (ptlim && s >= *ptlim)
+      s = realloc_buf(s,1, ptbuf,ptlim);
+    *s++ = *t++;
+  }
+  *s=0; *src=t; return s;
+}
+
+static char *
+readstring_i(char *s, char **ptbuf, char **ptlim)
+{
+  match('"'); s = translate(&analyseur,s, ptbuf,ptlim); match('"');
+  return s;
+}
+
+static GEN
+any_string()
+{
+  long n = 1, len = 16;
+  GEN res = cget1(len + 1, t_VEC);
+
+  while (*analyseur)
+  {
+    if (*analyseur == ')' || *analyseur == ';') break;
+    if (*analyseur == ',')
+      analyseur++;
+    else
+    {
+      res[n++] = (long)expr();
+      if (br_status) err(breaker,"here (print)");
+    }
+    if (n == len)
+    {
+      long newlen = len << 1;
+      GEN p1 = cget1(newlen + 1, t_VEC);
+      for (n = 1; n < len; n++) p1[n] = res[n];
+      res = p1; len = newlen;
+    }
+  }
+  setlg(res, n); return res;
+}
+
+/*  Read a "string" from src. Format then copy it, starting at s. Return
+ *  pointer to char following the end of the input string */
+char *
+readstring(char *src, char *s)
+{
+  match2(src, '"'); src++; s = translate(&src, s, NULL,NULL);
+  match2(src, '"'); return src+1;
+}
+
+/* return the first n0 chars of s as a GEN [s may not be 0-terminated] */
+static GEN
+_strtoGENstr(const char *s, long n0)
+{
+  long n = (n0+1+BYTES_IN_LONG) >> TWOPOTBYTES_IN_LONG;
+  GEN x = cgetg(n+1, t_STR);
+  char *t = GSTR(x);
+  strncpy(t, s, n0); t[n0] = 0; return x;
+}
+
+GEN
+STRtoGENstr(const char *s) { return _strtoGENstr(s, strlen(s)); }
+
+/* FIXME: obsolete, kept for gp2c compatibility */
+GEN
+strtoGENstr(char *s, long flag)
+{
+  GEN x;
+
+  if (flag) s = expand_tilde(s);
+  x = _strtoGENstr(s, strlen(s));
+  if (flag) free(s);
+  return x;
+}
+
 /********************************************************************/
 /**                                                                **/
 /**                          READ FUNCTIONS                        **/
@@ -888,7 +1099,7 @@ matcell(GEN p, matcomp *C)
   GEN *pt = &p;
   long c,r, tx;
   int full_col, full_row;
-  tx = full_col = full_row = 0; 
+  tx = full_col = full_row = 0;
   do {
     analyseur++; p = *pt; tx = typ(p);
     switch(tx)
@@ -973,13 +1184,14 @@ static GEN
 facteur(void)
 {
   const char *old = analyseur;
-  GEN x,p1;
-  int plus=1;
+  GEN x, p1;
+  int plus;
 
   switch(*analyseur)
   {
-    case '-': plus=0; /* fall through */
-    case '+': analyseur++; break;
+    case '-': analyseur++; plus = 0; break;
+    case '+': analyseur++; plus = 1; break;
+    default: plus = 1; break;
   }
   x = truc();
   if (br_status) return NULL;
@@ -1038,44 +1250,51 @@ _append(GEN **table, long *n, long *N)
 static GEN
 truc(void)
 {
-  long N,i,j,m,n,p;
-  GEN *table,p1;
+  long N, i, j, m, n, p;
+  GEN *table, z;
   char *old;
 
-  if (*analyseur == '!') /* NOT */
-  {
-    analyseur++; p1 = facteur();
-    if (br_status) err(breaker,"here (after !)");
-    return gcmp0(p1)? gun: gzero;
-  }
-  if (*analyseur == '\'') /* QUOTE */
-  {
-    const char* old;
-    entree *ep;
-    analyseur++; check_var_name();
-    old = analyseur; ep = entry();
-    switch(EpVALENCE(ep))
-    {
-      case EpVAR: case EpGVAR:
-        return (GEN)initial_value(ep);
-      default: err(varer1,old,mark.start);
-    }
-  }
-  if (*analyseur == '#') /* CARD */
-  {
-    analyseur++; p1 = facteur();
-    if (br_status) err(breaker,"here (after #)");
-    return stoi(glength(p1));
-  }
   if (isalpha((int)*analyseur)) return identifier();
-
-  if (*analyseur == '"') return strtoGENstr_t();
   if (isdigit((int)*analyseur) || *analyseur == '.') return constante();
-  switch(*analyseur++)
+
+  switch(*analyseur)
   {
-    case '(': p1=expr(); match(')'); return p1;
+    case '!': /* NOT */
+      analyseur++; z = facteur();
+      if (br_status) err(breaker,"here (after !)");
+      return gcmp0(z)? gun: gzero;
+
+    case ('\''): { /* QUOTE */
+      entree *ep;
+      analyseur++; check_var_name();
+      old = analyseur; ep = entry();
+      switch(EpVALENCE(ep))
+      {
+        case EpVAR: case EpGVAR:
+          return (GEN)initial_value(ep);
+        default: err(varer1,old,mark.start);
+      }
+    }
+    case '#': /* cardinal */
+      analyseur++; z = facteur();
+      if (br_status) err(breaker,"here (after #)");
+      return stoi(glength(z));
+
+    case '"': { /* string */
+      long n;
+      analyseur++; old = analyseur;
+      skipstring(); n = analyseur - old; /* do not count enclosing '"' */
+      n = (n+BYTES_IN_LONG) >> TWOPOTBYTES_IN_LONG;
+      z = cgetg(n+1, t_STR);
+      (void)translate(&old, GSTR(z), NULL,NULL);
+      return z;
+    }
+    case '(':
+      analyseur++;
+      z = expr(); match(')'); return z;
 
     case '[': /* constant array/vector */
+      analyseur++;
       if (*analyseur == ';' && analyseur[1] == ']')
 	{ analyseur += 2; return cgetg(1,t_MAT); } /* [;] */
 
@@ -1090,18 +1309,18 @@ truc(void)
         {
           long tx;
           if (*analyseur == '~') { analyseur++; tx=t_COL; } else tx=t_VEC;
-	  p1 = cgetg(n+1,tx);
-	  for (i=1; i<=n; i++) p1[i] = lcopy(table[i]);
+	  z = cgetg(n+1,tx);
+	  for (i=1; i<=n; i++) z[i] = lcopy(table[i]);
 	  break;
         }
 
 	case ';':
 	  m = n;
 	  do _append(&table, &n, &N); while (*analyseur++ != ']');
-	  p1 = cgetg(m+1,t_MAT); p = n/m + 1;
+	  z = cgetg(m+1,t_MAT); p = n/m + 1;
 	  for (j=1; j<=m; j++)
 	  {
-            GEN c = cgetg(p,t_COL); p1[j] = (long)c;
+            GEN c = cgetg(p,t_COL); z[j] = (long)c;
 	    for (i=j; i<=n; i+=m) *++c = lcopy(table[i]);
 	  }
 	  break;
@@ -1110,10 +1329,10 @@ truc(void)
           err(talker,"incorrect vector or matrix");
           return NULL; /* not reached */
       }
-      free(table); return p1;
+      free(table); return z;
 
     case '%':
-    old = analyseur-1;
+    old = analyseur; analyseur++;
     if (!GP_DATA) err(talker2,"history not available", old, mark.start);
     else
     {
@@ -1132,7 +1351,7 @@ truc(void)
 static GEN
 double_op()
 {
-  static long mun[] = { evaltyp(t_INT) | _evallg(3), 
+  static long mun[] = { evaltyp(t_INT) | _evallg(3),
                         evalsigne(-1)|evallgefint(3), 1 };
   char c = *analyseur;
   if (c == analyseur[1])
@@ -1261,171 +1480,6 @@ matrix_block(GEN p)
   return res;
 }
 
-static char*
-init_buf(long len, char **ptbuf, char **ptlim)
-{
-  char *buf = (char *)new_chunk(2 + len / sizeof(long));
-  *ptbuf = buf; *ptlim = buf + len; return buf;
-}
-
-static char*
-realloc_buf(char *bp, long len, char **ptbuf,char **ptlimit)
-{
-  char *buf = *ptbuf;
-  long newlen = ((*ptlimit - buf) + len) << 1;
-  long oldlen = bp - buf;
-
-  (void)init_buf(newlen, ptbuf, ptlimit);
-  memcpy(*ptbuf, buf, oldlen);
-  return *ptbuf + oldlen;
-}
-
-static char *
-expand_string(char *bp, char **ptbuf, char **ptlimit)
-{
-  char *tmp = NULL; /* -Wall */
-  long len = 0; /* -Wall */
-  int alloc = 1;
-
-  if (is_keyword_char(*analyseur))
-  {
-    char *s = analyseur;
-    do s++; while (is_keyword_char(*s));
-
-    if ((*s == '"' || *s == ',' || *s == ')') && !is_entry(analyseur))
-    { /* Do not create new user variable. Consider as a literal */
-      tmp = analyseur;
-      len = s - analyseur;
-      analyseur = s;
-      alloc = 0;
-    }
-  }
-
-  if (alloc)
-  {
-    gpmem_t av = avma;
-    GEN p1 = expr();
-    if (br_status) err(breaker,"here (expanding string)");
-    tmp = GENtostr0(p1, &DFLT_OUTPUT, &gen_output);
-    len = strlen(tmp); avma = av;
-  }
-  if (ptlimit && bp + len > *ptlimit)
-    bp = realloc_buf(bp, len, ptbuf,ptlimit);
-  memcpy(bp,tmp,len); /* ignore trailing \0 */
-  if (alloc) free(tmp);
-  return bp + len;
-}
-
-static char *
-translate(char **src, char *s, char **ptbuf, char **ptlim)
-{
-  char *t = *src;
-  while (*t)
-  {
-    while (*t == '\\')
-    {
-      switch(*++t)
-      {
-	case 'e':  *s='\033'; break; /* escape */
-	case 'n':  *s='\n'; break;
-	case 't':  *s='\t'; break;
-	default:   *s=*t; if (!*t) err(talker,"unfinished string");
-      }
-      t++; s++;
-    }
-    if (*t == '"')
-    {
-      if (t[1] != '"') break;
-      t += 2; continue;
-    }
-    if (ptlim && s >= *ptlim)
-      s = realloc_buf(s,1, ptbuf,ptlim);
-    *s++ = *t++;
-  }
-  *s=0; *src=t; return s;
-}
-
-static char *
-readstring_i(char *s, char **ptbuf, char **ptlim)
-{
-  match('"'); s = translate(&analyseur,s, ptbuf,ptlim); match('"');
-  return s;
-}
-
-static GEN
-any_string()
-{
-  long n = 0, len = 16;
-  GEN res = new_chunk(len + 1);
-
-  while (*analyseur)
-  {
-    if (*analyseur == ')' || *analyseur == ';') break;
-    if (*analyseur == ',')
-      analyseur++;
-    else
-    {
-      res[n++] = (long)expr();
-      if (br_status) err(breaker,"here (print)");
-    }
-    if (n == len)
-    {
-      long newlen = len << 1;
-      GEN p1 = new_chunk(newlen + 1);
-      for (n = 0; n < len; n++) p1[n] = res[n];
-      res = p1; len = newlen;
-    }
-  }
-  res[n] = 0; /* end the sequence with NULL */
-  return res;
-}
-
-/*  Read a "string" from src. Format then copy it, starting at s. Return
- *  pointer to the \0 which terminates the string.
- */
-char *
-readstring(char *src, char *s)
-{
-  match2(src, '"'); src++; s = translate(&src, s, NULL,NULL);
-  match2(src, '"'); return s;
-}
-
-static GEN
-strtoGENstr_t()
-{
-  char *old = analyseur;
-  long n;
-  GEN x;
-
-  skipstring(); n = analyseur-old - 1; /* don't count the enclosing '"' */
-  old++; /* skip '"' */
-  n = (n+BYTES_IN_LONG) >> TWOPOTBYTES_IN_LONG;
-  x = cgetg(n+1, t_STR);
-  (void)translate(&old, GSTR(x), NULL,NULL);
-  return x;
-}
-
-/* return the first n0 chars of s as a GEN [s may not be 0terminated] */
-static GEN
-_strtoGENstr(char *s, long n0)
-{
-  long n = (n0+1+BYTES_IN_LONG) >> TWOPOTBYTES_IN_LONG;
-  GEN x = cgetg(n+1, t_STR);
-  char *t = GSTR(x);
-  strncpy(t, s, n0); t[n0] = 0; return x;
-}
-
-GEN
-strtoGENstr(char *s, long flag)
-{
-  GEN x;
-
-  if (flag) s = expand_tilde(s);
-  x = _strtoGENstr(s, strlen(s));
-  if (flag) free(s);
-  return x;
-}
-
 /* x = gzero: no default value, otherwise a t_STR, formal expression for
  * default argument. Evaluate and return. */
 static GEN
@@ -1573,7 +1627,7 @@ check_args()
     if (*analyseur == '=')
     {
       char *old = ++analyseur;
-      gpmem_t av = avma;
+      pari_sp av = avma;
       skipexpr();
       cell[1] = lclone(_strtoGENstr(old, analyseur-old));
       avma = av;
@@ -1619,7 +1673,7 @@ num_deriv(void *call, GEN argvec[])
 {
   GEN eps,a,b, y, x = argvec[0];
   long fpr, pr, l, e, ex;
-  gpmem_t av = avma;
+  pari_sp av = avma;
   if (!is_const_t(typ(x)))
   {
     a = do_call(call, x, argvec);
@@ -1646,7 +1700,7 @@ num_derivU(GEN p, GEN *arg, gp_args *f)
 {
   GEN eps,a,b, x = *arg;
   long fpr, pr, l, e, ex;
-  gpmem_t av = avma;
+  pari_sp av = avma;
 
   if (!is_const_t(typ(x)))
   {
@@ -1677,7 +1731,7 @@ static GEN
 identifier(void)
 {
   long m, i, matchcomma, deriv;
-  gpmem_t av;
+  pari_sp av;
   char *ch1;
   entree *ep;
   GEN res, newfun, ptr;
@@ -1894,8 +1948,8 @@ identifier(void)
               case 'G':
               case '&':
               case 'I':
-              case 'V': argvec[i++]=DFT_GEN; s++; break;
-              case 'n': argvec[i++]=DFT_VAR; s++; break;
+              case 'V': matchcomma=1; argvec[i++]=DFT_GEN; s++; break;
+              case 'n': matchcomma=1; argvec[i++]=DFT_VAR; s++; break;
               default:
                 oldanalyseur = analyseur;
                 analyseur = s; matchcomma = 0;
@@ -2041,7 +2095,7 @@ identifier(void)
           analyseur=ch1; ep = entry();
           if (*analyseur == '=')
           {
-            gpmem_t av=avma; analyseur++;
+            pari_sp av=avma; analyseur++;
             res = expr();
             if (br_status) err(breaker,"here (defining global var)");
             changevalue(ep, res); avma=av;
@@ -2094,7 +2148,6 @@ identifier(void)
           { /* user supplied */
             match_comma();
             arglist[i] = expr();
-            skipdecl(); /* we'd be redefining fun, but don't know it yet */
             if (br_status) err(breaker,"here (reading function args)");
           }
         }
@@ -2117,6 +2170,8 @@ identifier(void)
       analyseur = ch1-1; /* points to '(' */
 
       free_args((gp_args*)ep->args);
+      free(ep->args);
+      ep->valence = EpNEW;
     /* Fall through */
 
     case EpNEW: /* new function */
@@ -2130,7 +2185,7 @@ identifier(void)
       /* checking arguments */
       match('('); ch1 = analyseur;
       narg = check_args(); nloc = 0;
-      match(')'); 
+      match(')');
       /* Dirty, but don't want to define a local() function */
       if (*analyseur != '=' && strcmp(ep->name, "local") == 0)
         err(talker2, "local() bloc must appear before any other expression",
@@ -2185,7 +2240,7 @@ identifier(void)
       ep->value = (void *)ptr;
       ep->valence = EpUSER;
       check_new_fun=NULL;
-      avma = (gpmem_t)tmpargs; return gnil;
+      avma = (pari_sp)tmpargs; return gnil;
     }
   }
   err(valencer1); return NULL; /* not reached */
@@ -2200,13 +2255,19 @@ number(long *nb)
   return m;
 }
 
-static GEN
-constante()
+long
+pow10(int n)
 {
   static long pw10[] = { 1, 10, 100, 1000, 10000, 100000, 1000000,
                         10000000, 100000000, 1000000000 };
+  return pw10[n];
+}
+
+static GEN
+constante()
+{
   long i, l, m, n = 0, nb;
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN z,y;
 
   y = stoi(number(&nb)); i = 0;
@@ -2214,7 +2275,7 @@ constante()
   {
     if (++i == 4) { avma = av; i = 0; } /* HACK gerepile */
     m = number(&nb);
-    y = addsmulsi(m, pw10[nb], y);
+    y = addsmulsi(m, pow10(nb), y);
   }
   switch(*analyseur)
   {
@@ -2228,7 +2289,7 @@ constante()
       {
         if (++i == 4) { avma = av; i = 0; } /* HACK gerepile */
         m = number(&nb); n -= nb;
-        y = addsmulsi(m, pw10[nb], y);
+        y = addsmulsi(m, pow10(nb), y);
       }
       if (*analyseur != 'E' && *analyseur != 'e')
       {
@@ -2248,23 +2309,21 @@ constante()
       if (nb > 8) err(talker2,"exponent too large",old,mark.start);
       if (!signe(y))
       {
-        avma = av; y = cgetr(3);
+        avma = av;
         n = (n > 0)? (long)(n/L2SL10): (long)-((-n)/L2SL10 + 1);
-        y[1] = evalsigne(0) | evalexpo(n);
-        y[2] = 0; return y;
+        return realzero_bit(n);
       }
     }
   }
-  l=lgefint(y); if (l<prec) l=prec;
+  l = lgefint(y); if (l < (long)prec) l = (long)prec;
+  z = itor(y, l);
   if (n)
   {
-    (void)new_chunk(l); /* HACK: mulrr and divrr need exactly l words */
-    z = itor(y, l);
-    y = gpowgs(stor(10,l), labs(n));
-    avma = av; /* hidden gerepile */
-    return n > 0 ?  mulrr(z,y) : divrr(z,y);
+    y = rpowsi(10UL, stoi(labs(n)), l+1); /* 10^|n| */
+    z = (n > 0)? mulrr(z,y): divrr(z,y);
+    z = gerepileuptoleaf(av, z);
   }
-  return itor(y, l);
+  return z;
 }
 
 /********************************************************************/
@@ -2347,31 +2406,29 @@ installep(void *f, char *name, int len, int valence, int add, entree **table)
 long
 manage_var(long n, entree *ep)
 {
-  static long max_avail = MAXVARN; /* first user variable not yet used */
+  static long max_avail = MAXVARN; /* max variable not yet used */
   static long nvar; /* first GP free variable */
   long var;
   GEN p;
 
-  if (n) /* special behaviour */
-  {
-    switch(n)
-    {
-      case 2: return nvar=0;
-      case 3: return nvar;
-      case 4: return max_avail;
-      case 5:
+  switch(n) {
+      case manage_var_init: return nvar=0;
+      case manage_var_next: return nvar;
+      case manage_var_max_avail: return max_avail;
+      case manage_var_pop:
       {
         long v = (long)ep;
         if (v != nvar-1) err(talker,"can't pop gp variable");
         setlg(polvar, nvar);
         return --nvar;
       }
-    }
-
-    /* user wants to delete one of his/her/its variables */
-    if (max_avail == MAXVARN-1) return 0; /* nothing to delete */
-    free(polx[++max_avail]); /* frees both polun and polx */
-    return max_avail+1;
+      case manage_var_delete:
+	/* user wants to delete one of his/her/its variables */
+	if (max_avail == MAXVARN-1) return 0; /* nothing to delete */
+	free(polx[++max_avail]); /* frees both polun and polx */
+	return max_avail+1;
+      case manage_var_create: break;
+      default: err(talker, "panic");
   }
 
   if (nvar == max_avail) err(talker2,"no more variables available",
@@ -2383,7 +2440,7 @@ manage_var(long n, entree *ep)
   }
   else
   {
-    p = (GEN) gpmalloc(7*sizeof(long));
+    p = (GEN) gpmalloc(SIZEOF_VAR_POLS);
     var=max_avail--;
   }
 
@@ -2408,7 +2465,7 @@ manage_var(long n, entree *ep)
 long
 fetch_var(void)
 {
-  return manage_var(0,NULL);
+  return manage_var(manage_var_create,NULL);
 }
 
 entree *
@@ -2420,16 +2477,16 @@ fetch_named_var(char *s, int doerr)
     if (doerr) err(talker,"identifier already in use: %s", s);
     return ep;
   }
-  ep = installep(NULL,s,strlen(s),EpVAR, 7*sizeof(long),
+  ep = installep(NULL,s,strlen(s),EpVAR, SIZEOF_VAR_POLS,
                  functions_hash + hashvalue(s));
-  (void)manage_var(0,ep); return ep;
+  (void)manage_var(manage_var_create,ep); return ep;
 }
 
 long
 fetch_user_var(char *s)
 {
   entree *ep = is_entry(s);
-  gpmem_t av;
+  pari_sp av;
   GEN p1;
 
   if (ep)
@@ -2448,14 +2505,14 @@ fetch_user_var(char *s)
 void
 delete_named_var(entree *ep)
 {
-  (void)manage_var(5, (entree*)varn(initial_value(ep)));
+  (void)manage_var(manage_var_pop, (entree*)varn(initial_value(ep)));
   kill0(ep);
 }
 
 long
 delete_var(void)
 {
-  return manage_var(1,NULL);
+  return manage_var(manage_var_delete,NULL);
 }
 
 void
@@ -2464,9 +2521,9 @@ name_var(long n, char *s)
   entree *ep;
   char *u;
 
-  if (n < manage_var(3,NULL))
+  if (n < manage_var(manage_var_next,NULL))
     err(talker, "renaming a GP variable is forbidden");
-  if (n > MAXVARN)
+  if (n > (long)MAXVARN)
     err(talker, "variable number too big");
 
   ep = (entree*)gpmalloc(sizeof(entree) + strlen(s) + 1);
@@ -2497,10 +2554,10 @@ entry(void)
   if (*analyseur == '(')
     { n=0; val=EpNEW; }
   else
-    { n=7*sizeof(long); val=EpVAR; }
+    { n=SIZEOF_VAR_POLS; val=EpVAR; }
   ep = installep(NULL,old,len,val,n, functions_hash + hash);
 
-  if (n) (void)manage_var(0,ep); /* Variable */
+  if (n) (void)manage_var(manage_var_create, ep); /* Variable */
   return ep;
 }
 
@@ -2523,7 +2580,7 @@ doskipseq(char *c, int strict)
     if (strict) err(talker2,"unused characters", analyseur, c);
     L = term_width();
     n = 2 * L - (17+19+1); /* Warning + unused... + . */
-    if (strlen(analyseur) > n)
+    if ((long)strlen(analyseur) > n)
     {
       s = gpmalloc(n + 1);
       n -= 5;
@@ -2537,11 +2594,11 @@ doskipseq(char *c, int strict)
   analyseur = olds;
 }
 
+/* analyseur points on the character following the starting " */
 /* skip any number of concatenated strings */
 static void
 skipstring()
 {
-  match('"');
   while (*analyseur)
     switch (*analyseur++)
     {
@@ -2693,22 +2750,27 @@ skiptruc(void)
   long i,m,n;
   char *old;
 
-  switch(*analyseur)
-  {
-    case '"': skipstring(); return;
-    case '!': case '#': analyseur++; skipfacteur(); return;
-    case '&': case '\'':
-      analyseur++; check_var_name();
-      (void)skipentry(); return;
-  }
   if (isalpha((int)*analyseur)) { skipidentifier(); return; }
   if (isdigit((int)*analyseur) || *analyseur== '.') { skipconstante(); return; }
-  switch(*analyseur++)
+  switch(*analyseur)
   {
+    case '"':
+      analyseur++; skipstring(); return;
+
+    case '!':
+    case '#':
+      analyseur++; skipfacteur(); return;
+
+    case '&':
+    case '\'':
+      analyseur++; check_var_name();
+      (void)skipentry(); return;
+
     case '(':
-      skipexpr(); match(')'); return;
+      analyseur++; skipexpr(); match(')'); return;
+
     case '[':
-      old = analyseur-1;
+      old = analyseur; analyseur++;
       if (*analyseur == ';' && analyseur[1] == ']')  /* [;] */
         { analyseur+=2; return; }
       n = 0;
@@ -2717,10 +2779,10 @@ skiptruc(void)
 	do { n++; skipexpr(); old=analyseur; } while (*analyseur++ == ',');
 	analyseur--;
       }
-      switch (*analyseur)
+      switch (*analyseur++)
       {
-	case ']': analyseur++; return;
-	case ';': analyseur++;
+	case ']': return;
+	case ';':
           for (m=2; ; m++)
           {
             for (i=1; i<n; i++) { skipexpr(); match(','); }
@@ -2732,7 +2794,9 @@ skiptruc(void)
 	default:
 	  err(talker2,"; or ] expected",old,mark.start);
       }
+
     case '%':
+      analyseur++;
       if (*analyseur == '`') { while (*++analyseur == '`') /*empty*/; return; }
       (void)number(&n); return;
   }
@@ -2786,6 +2850,7 @@ skipidentifier(void)
     }
     analyseur++;
 
+    if (*s == 'v' || *s == 'l') s++;
     /* Optimized for G and p. */
     matchcomma = 0;
     while (*s == 'G') { match_comma(); skipexpr(); s++; }
@@ -2804,7 +2869,7 @@ skipidentifier(void)
         match_comma();
         while (*analyseur)
         {
-          if (*analyseur == '"') skipstring();
+          if (*analyseur == '"') { analyseur++; skipstring(); }
           if (*analyseur == ',' || *analyseur == ')') break;
           analyseur++;
         }
@@ -2815,7 +2880,7 @@ skipidentifier(void)
         {
           while (*analyseur)
           {
-            if (*analyseur == '"') skipstring();
+            if (*analyseur == '"') { analyseur++; skipstring(); }
             if (*analyseur == ')') break;
             if (*analyseur == ',') analyseur++;
             else skipexpr();
@@ -2826,7 +2891,7 @@ skipidentifier(void)
 
         while (*analyseur)
         {
-          if (*analyseur == '"') skipstring();
+          if (*analyseur == '"') { analyseur++; skipstring(); }
           if (*analyseur == ',' || *analyseur == ')') break;
           skipexpr();
         }
@@ -2837,7 +2902,7 @@ skipidentifier(void)
       case '&': match_comma(); match('&'); check_matcell(); break;
       case 'V': match_comma(); check_var(); break;
 
-      case 'p': case 'P': case 'l': case 'v': case 'f': case 'x':
+      case 'p': case 'P': case 'f': case 'x':
         break;
 
       case 'D':
