@@ -1,4 +1,4 @@
-/* $Id: es.c,v 1.94 2002/09/12 13:28:14 karim Exp $
+/* $Id: es.c,v 1.110 2003/02/08 18:03:36 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -20,9 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*******************************************************************/
 #include "pari.h"
 #include "anal.h"
-extern GEN confrac(GEN x); /* should be static here, but use hiremainder */
-extern GEN convi(GEN x);
-char * type_name(long t);
+extern char *type_name(long t);
 
 void
 hit_return(void)
@@ -54,7 +52,7 @@ filtre0(filtre_t *F)
 
   if (F->more_input == 1) F->more_input = 0;
 
-  if (! F->in_comment)
+  if (! (F->in_comment | F->in_string))
   {
     while (isspace((int)*s)) s++; /* Skip space */
     if (*s == LBRACE) { s++; F->more_input = 2; F->wait_for_brace = 1; }
@@ -191,7 +189,7 @@ normalOutC(char c)
   if (logfile) putc(c, logfile);
 }
 static void
-normalOutS(char *s)
+normalOutS(const char *s)
 {
   fputs(s, pari_outfile);
   if (logfile) { fputs(s, logfile); }
@@ -211,7 +209,7 @@ normalErrC(char c)
   if (logfile) putc(c, logfile);
 }
 static void
-normalErrS(char *s)
+normalErrS(const char *s)
 {
   fputs(s, errfile);
   if (logfile) fputs(s, logfile);
@@ -237,7 +235,7 @@ void
 pariputc(char c) { pariOut->putch(c); }
 
 void
-pariputs(char *s) { pariOut->puts(s); }
+pariputs(const char *s) { pariOut->puts(s); }
 
 void
 pariflush(void) { pariOut->flush(); }
@@ -247,10 +245,11 @@ flusherr(void) { pariErr->flush(); }
 
 /* format is standard printf format, except %Z is a GEN */
 void
-vpariputs(char* format, va_list args)
+vpariputs(const char* format, va_list args)
 {
   long nb = 0, bufsize = 1023;
-  char *buf, *str, *s, *f = format;
+  const char *f = format;
+  char *buf, *str, *s, *t;
   
   /* replace each %Z (2 chars) by braced address format (8 chars) */
   s = str = gpmalloc(strlen(format)*4 + 1);
@@ -282,25 +281,25 @@ vpariputs(char* format, va_list args)
   buf = gpmalloc(bufsize);
   (void)vsprintf(buf,str,args); /* pray it does fit */
 #endif
-  f = s = buf;
+  t = s = buf;
   if (nb)
-    while ( *f )
+    while (*t)
     {
-      if (*f == '\003' && f[21] == '\003')
+      if (*t == '\003' && t[21] == '\003')
       {
-        *f = 0; f[21] = 0; /* remove the bracing chars */
-        pariOut->puts(s); bruteall((GEN)atol(f+1),'g',-1,1);
-        f += 22; s = f;
+        *t = 0; t[21] = 0; /* remove the bracing chars */
+        pariOut->puts(s); bruteall((GEN)atol(t+1),'g',-1,1);
+        t += 22; s = t;
         if (!--nb) break; 
       }
       else
-        f++;
+        t++;
     }
   pariOut->puts(s); free(buf); free(str);
 }
 
 void
-pariputsf(char *format, ...)
+pariputsf(const char *format, ...)
 {
   va_list args;
 
@@ -443,7 +442,7 @@ putc80(char c)
 }
 #undef MAX_WIDTH
 static void
-puts80(char *s)
+puts80(const char *s)
 {
   while (*s) putc80(*s++);
 }
@@ -477,7 +476,7 @@ putc_lim_lines(char c)
   col_index++; normalOutC(c);
 }
 static void
-puts_lim_lines(char *s)
+puts_lim_lines(const char *s)
 {
   long i,len;
   if (lin_index > max_lin) return;
@@ -628,9 +627,9 @@ errstr_putc(char c) { str_putc(ErrStr, c); }
   str->len += len; \
 }
 static void
-outstr_puts(char *s) { str_puts(OutStr, s); }
+outstr_puts(const char *s) { str_puts(OutStr, s); }
 static void
-errstr_puts(char *s) { str_puts(ErrStr, s); }
+errstr_puts(const char *s) { str_puts(ErrStr, s); }
 
 static void
 outstr_flush(void) { /* empty */ }
@@ -639,7 +638,7 @@ PariOUT pariErr2Str = {errstr_putc, errstr_puts, outstr_flush, NULL};
 #undef STEPSIZE
 
 char *
-pari_strdup(char *s)
+pari_strdup(const char *s)
 {
   int n = strlen(s)+1;
   char *t = gpmalloc(n);
@@ -667,6 +666,61 @@ GENtostr0(GEN x, pariout_t *T, void (*do_out)(GEN, pariout_t*))
 char *
 GENtostr(GEN x) { return GENtostr0(x, NULL, &gen_output); }
 
+/* Returns gpmalloc()ed string */
+char *
+GENtoTeXstr(GEN x) {
+  pariout_t T = DFLT_OUTPUT;
+
+  T.prettyp = f_TEX;
+  T.fieldw = 0;
+  return GENtostr0(x, &T, &gen_output);
+}
+
+/* see print0(). Returns gpmalloc()ed string */
+char *
+pGENtostr(GEN g, long flag) {
+  pariout_t T = DFLT_OUTPUT;
+  pari_sp av = avma;
+  char *t, *t2;
+  long i, tlen = 0, l = lg(g);
+  GEN Ls, Ll;
+
+  T.prettyp = flag;
+  /* frequent special case */
+  if (l == 2) return GENtostr0((GEN)g[1], &T, &gen_output);
+
+  Ls = cgetg(l, t_VEC);
+  Ll = cgetg(l, t_VECSMALL);
+  for (i = 1; i < l; i++)
+  {
+    char *s = GENtostr0((GEN)g[i], &T, &gen_output);
+    Ls[i] = (long)s;
+    Ll[i] = strlen(s);
+    tlen += Ll[i];
+  }
+  t2 = t = gpmalloc(tlen + 1);
+  for (i = 1; i < l; i++)
+  {
+    strcpy(t2, (char*)Ls[i]);
+    t2 += Ll[i];
+    free((void*)Ls[i]);
+  }
+  avma = av; return t;
+}
+GEN Str0(GEN g, long flag) {
+  char *t = pGENtostr(g, flag);
+  GEN z = STRtoGENstr(t);
+  free(t); return z;
+}
+GEN Str(GEN g)    { return Str0(g, f_RAW); }
+GEN Strtex(GEN g) { return Str0(g, f_TEX); }
+GEN
+Strexpand(GEN g) {
+  char *s = pGENtostr(g, f_RAW), *t = expand_tilde(s);
+  GEN z = STRtoGENstr(t);
+  free(t); free(s); return z;
+}
+
 /********************************************************************/
 /**                                                                **/
 /**                         WRITE AN INTEGER                       **/
@@ -675,8 +729,41 @@ GENtostr(GEN x) { return GENtostr0(x, NULL, &gen_output); }
 #define putsigne_nosp(x) pariputc((x>0)? '+' : '-')
 #define putsigne(x) pariputs((x>0)? " + " : " - ")
 #define sp_sign_sp(T,x) ((T)->sp? putsigne(x): putsigne_nosp(x))
-#define sp(T) do { if ((T)->sp) pariputc(' '); } while(0);
 #define comma_sp(T)     ((T)->sp? pariputs(", "): pariputc(','))
+#define sp(T) STMT_START { if ((T)->sp) pariputc(' '); } STMT_END
+
+/* convert integer --> base 10^9 [not memory clean] */
+static long *
+convi(GEN x, long *l)
+{
+  pari_sp av, lim;
+  long lz = 3 + (long)((lgefint(x)-2) * (BITS_IN_LONG / (9*L2SL10)));
+  GEN z, zd;
+
+  zd = z = new_chunk(lz);
+  av = avma; lim = stack_lim(av,1);
+  for(;;)
+  {
+    x = diviu_rem(x, 1000000000, (ulong*)zd); zd++;
+    if (!signe(x)) { if (l) *l = zd - z; return zd; }
+    if (low_stack(lim, stack_lim(av,1))) x = gerepileuptoint(av, x);
+  }
+}
+
+/* # of decimal digits, assume l > 0 */
+static long
+numdig(long l)
+{
+  if (l < 100000)
+  {
+    if (l < 100)    return (l < 10)? 1: 2;
+    if (l < 10000)  return (l < 1000)? 3: 4;
+    return 5;
+  }
+  if (l < 10000000)   return (l < 1000000)? 6: 7;
+  if (l < 1000000000) return (l < 100000000)? 8: 9;
+  return 10;
+}
 
 static void
 blancs(long nb) { while (nb-- > 0) pariputc(' '); }
@@ -684,91 +771,49 @@ blancs(long nb) { while (nb-- > 0) pariputc(' '); }
 static void
 zeros(long nb)  { while (nb-- > 0) pariputc('0'); }
 
-static long
-coinit(long x)
-{
-  char cha[10], *p = cha + 9;
-
-  *p = 0;
-  do { *--p = x%10 + '0'; x /= 10; } while (x);
-  pariputs(p); return 9 - (p - cha);
-}
-
-/* as above, printing leading 0s, return # significant digits printed
- * print at most dec significant digits */
-static long
-coinit2(long x, long dec)
-{
-  char cha[10], *p = cha + 9;
-  int i = 0;
-
-  for (*p = 0; p > cha; x /= 10) *--p = x%10 + '0';
-  while (cha[i] == '0') i++;
-  i = 9-i; /* # significant digits to print */
-  if (i > dec) { i = dec; cha[dec] = 0; }
-  pariputs(cha); return i;
-}
-
 static void
-comilieu(long x)
+copart(char *s, long x, long start)
 {
-  char cha[10], *p = cha + 9;
-
-  for (*p = 0; p > cha; x /= 10) *--p = x%10 + '0';
-  pariputs(cha);
+  char *p = s + start;
+  for ( ; p > s; x /= 10) *--p = x%10 + '0';
 }
-
 static void
-cofin(long x, long decim)
-{
-  char cha[10], *p = cha + 9;
+comid(char *s, long x) { copart(s,x,9); }
 
-  for (; p > cha; x /= 10) *--p = x%10 + '0';
-  cha[decim] = 0; pariputs(cha);
+/* convert abs(x) != 0 to str. Prepend '-' if (minus) */
+static char *
+itostr(GEN x, int minus)
+{
+  long l, d, *res = convi(x, &l);
+  char *s = (char*)new_chunk(l + minus + 1), *t = s;
+
+  if (minus) *t++ = '-';
+  d = numdig(*--res);
+  copart(t, *res, d); t += d;
+  while (--l > 0) { comid(t, *--res); t += 9; }
+  *t = 0; return s;
 }
 
-static long
-nbdch(long l)
-{
-  if (l<100000)
-  {
-    if (l<10) return 1;
-    if (l<100) return 2;
-    if (l<1000) return 3;
-    if (l<10000) return 4;
-    return 5;
-  }
-  if (l<1000000) return 6;
-  if (l<10000000) return 7;
-  if (l<100000000) return 8;
-  if (l<1000000000) return 9;
-  return 10; /* not reached */
-}
-
-/* write int x > 0 */
+/* x != 0 integer, write abs(x). Prepend '-' if (minus) */
 static void
-wr_intpos(GEN x)
+wr_intsgn(GEN x, int minus)
 {
-  long *res = convi(x);
-  (void)coinit(*--res); while (*--res >= 0) comilieu(*res);
+  pari_sp av = avma;
+  pariputs( itostr(x, minus) ); avma = av;
 }
 
 /* write int. T->fieldw: field width (pad with ' ') */
 static void
 wr_int(pariout_t *T, GEN x, int nosign)
 {
-  long *res,*re,i, sx = signe(x);
-  int minus;
+  long sx = signe(x);
+  pari_sp av = avma;
+  char *s;
 
   if (!sx) { blancs(T->fieldw - 1); pariputc('0'); return; }
-  re = res = convi(x);
-  i = nbdch(*--re); while (*--re >= 0) i+=9;
-  minus = (sx < 0 && !nosign);
-  if (minus) i++;
-
-  blancs(T->fieldw - i);
-  if (minus) pariputc('-');
-  (void)coinit(*--res); while (*--res >= 0) comilieu(*res);
+  s = itostr(x, sx < 0 && !nosign);
+  blancs( T->fieldw - strlen(s) );
+  pariputs(s); avma = av;
 }
 
 static void
@@ -788,70 +833,100 @@ wr_vecsmall(pariout_t *T, GEN g)
 /**                        WRITE A REAL NUMBER                     **/
 /**                                                                **/
 /********************************************************************/
-static void wr_exp(pariout_t *T, GEN x);
+extern long pow10(int n);
+extern GEN rpowsi(ulong a, GEN n, long prec);
+
+/* e binary exponent, return exponent in base ten */
+static long
+ex10(long e) { return (long) ((e >= 0)? e*L2SL10: -(-e*L2SL10)-1); }
+
+/* sss.ttt, assume 'point' < strlen(s) */
+static void
+wr_dec(char *s, long point)
+{
+  char *t = s + point, save = *t;
+  *t = 0;    pariputs(s); pariputc('.'); /* integer part */
+  *t = save; pariputs(t);
+}
+/* a.bbb En*/
+static void
+wr_exp(pariout_t *T, char *s, long n)
+{
+  wr_dec(s, 1); sp(T); pariputsf("E%ld", n);
+}
+
+static void
+round_up(long *resd, long n, long *res)
+{
+  *resd += n;
+  while (*resd >= 1000000000 && resd < res) { *resd++ = 0; *resd += 1; }
+}
 
 /* assume x != 0 and print |x| in floating point format */
 static void
-wr_float(pariout_t *T, GEN x)
+wr_float(pariout_t *T, GEN x, int f_format)
 {
-  long *res, ex,s,d,e,decmax, dec = T->sigd;
-  GEN p1;
-
-  if (dec>0) /* round if needed */
-  {
-    GEN arrondi = cgetr(3);
-    ex = expo(x) - (long)((((double)BITS_IN_LONG)/pariK)*dec+2);
-    arrondi[1] = (x[1] & ~EXPOBITS) | evalexpo(ex);
-    arrondi[2] = x[2]; x = addrr(x,arrondi);
+  long beta, l, ldec, dec0, decdig, d, dif, lx = lg(x), dec = T->sigd;
+  GEN z, res, resd;
+  char *s, *t;
+  
+  dif =  bit_accuracy(lx) - expo(x);
+  if (dif <= 0) f_format = 0; /* write in E format */
+  if (dec > 0)
+  { /* reduce precision if possible */
+    l = 3 + (dec * pariK1);
+    if (lx > l) { x = rtor(x, l); lx = l; }
   }
-  ex = expo(x); e = bit_accuracy(lg(x)); /* significant bits */
-  if (ex >= e) { wr_exp(T,x); return; }
-  decmax = (long) (e * L2SL10); /* significant digits */
-  if ((ulong)decmax < (ulong)dec) dec = decmax; /* Hack: includes dec < 0 */
-
-/* integer part */
-  p1 = gcvtoi(x,&e); s = signe(p1);
-  if (e > 0) err(bugparier,"wr_float");
-  if (!s) { pariputc('0'); d=0; }
-  else
+  
+  beta = ex10(dif);
+  if (beta)
   {
-    x = subri(x,p1); setsigne(p1,1);
-    res = convi(p1); d = coinit(*--res);
-    while (*(--res) >= 0) { d += 9; comilieu(*res); }
+    z = rpowsi(10UL, stoi(labs(beta)), lx+1);
+    z = (beta > 0)? mulrr(x, z): divrr(x, z);
+    setsigne(z, 1);
   }
-  pariputc('.');
+  else z = mpabs(x);
 
-/* fractional part: 0 < x < 1 */
-  if (!signe(x))
+  /* x ~ z / 10^beta, z in N */
+  z = grndtoi(z, &l); /* l is junk */
+  res = convi(z, &ldec);
+  resd = res - 1; /* leading word */
+  dec0 = numdig(*resd);
+  decdig = 9 * (ldec-1) + dec0; /* number of significant decimal digits in z */
+
+  /* Round properly; leading word may overflow to 10^9 after rounding */
+  if (dec < 0 || decdig < dec)
+    dec = decdig;
+  else if (dec < dec0) /* ==> 0 < dec < 9 */
   {
-    dec -= d; if (dec>0) zeros(dec);
-    return;
+    long p10 = pow10(dec0 - dec);
+    if (*resd % p10 > (p10>>1)) *resd += p10; /* round up */
+  }
+  else if (dec < decdig)
+  {
+    l = dec - dec0; /* skip leading word */
+    d = l % 9;
+    resd -= l / 9;
+    if (d)
+    { /* cut resd[-1] to first d digits when printing */
+      long p10 = pow10(9 - d);
+      if (*--resd % p10 > (p10>>1)) round_up(resd, p10, res);
+    }
+    else if ((ulong)resd[-1] > 500000000) round_up(resd, 1, res);
   }
 
-  res = confrac(x);
-  if (!s)
-  {
-    while (!*res) { res++; pariputs("000000000"); }
-    d = coinit2(*res++, dec);
-  }
+  s = t = (char*)new_chunk(decdig + 1);
+  l = ldec;
+  d = numdig(*--res);
+  copart(t, *res, d); t += d;
+  while (--l > 0) { comid(t, *--res); t += 9; }
+  s[dec] = 0;
 
-  /* d = # significant digits already printed */
-  dec -= d;
-  while (dec>8) { comilieu(*res++); dec -= 9; }
-  if (dec>0) cofin(*res,dec);
-}
-
-/* as above in exponential format */
-static void
-wr_exp(pariout_t *T, GEN x)
-{
-  GEN dix = stor(10, lg(x)+1);
-  long e = expo(x);
-
-  e = (e>=0)? (long)(e*L2SL10): (long)(-(-e*L2SL10)-1);
-  if (e) x = mulrr(x, gpowgs(dix,-e));
-  if (absr_cmp(x, dix) >= 0) { x = divrr(x,dix); e++; }
-  wr_float(T,x); sp(T); pariputsf("E%ld",e);
+  decdig = d + 9*(ldec-1); /* recompute: d may be 1 more */
+  dif = decdig - beta; /* # of decimal digits in integer part */
+  if (!f_format || dif > dec) wr_exp(T,s, dif-1);
+  else if (dif > 0) wr_dec(s, dif);
+  else { pariputs("0."); zeros(-dif); pariputs(s); }
 }
 
 /* Write real number x.
@@ -862,34 +937,29 @@ wr_exp(pariout_t *T, GEN x)
 static void
 wr_real(pariout_t *T, GEN x, int nosign)
 {
-  gpmem_t ltop;
+  pari_sp av;
   long sx = signe(x), ex = expo(x);
 
   if (!sx) /* real 0 */
   {
     if (T->format == 'f')
     {
-      long d, dec = T->sigd;
+      long dec = T->sigd;
       if (dec < 0)
       {
-        d = 1+((-ex)>>TWOPOTBITS_IN_LONG);
-        if (d < 0) d = 0;
-        dec = (long)(pariK*d);
+        long d = 1 + ((-ex)>>TWOPOTBITS_IN_LONG);
+        dec = (d <= 0)? 0: (long)(pariK*d);
       }
       pariputs("0."); zeros(dec);
     }
     else
-    {
-      ex = (ex>=0)? (long)(ex*L2SL10): (long)(-(-ex*L2SL10)-1);
-      pariputsf("0.E%ld", ex+1);
-    }
+      pariputsf("0.E%ld", ex10(ex) + 1);
     return;
   }
   if (!nosign && sx < 0) pariputc('-'); /* print sign if needed */
-  ltop = avma;
-  if ((T->format == 'g' && ex>=-32)
-    || T->format == 'f') wr_float(T,x); else wr_exp(T,x);
-  avma = ltop;
+  av = avma;
+  wr_float(T,x, (T->format == 'g' && ex >= -32) || T->format == 'f');
+  avma = av;
 }
 
 /********************************************************************/
@@ -987,7 +1057,7 @@ voir2(GEN x, long nb, long bl)
   {
     case t_INTMOD: case t_POLMOD:
     {
-      char *s = (tx==t_INTMOD)? "int = ": "pol = ";
+      const char *s = (tx==t_INTMOD)? "int = ": "pol = ";
       if (isonstack(x[1])) blancs(bl); else { blancs(bl-2); pariputs("* "); }
       pariputs("mod = "); voir2((GEN)x[1],nb,bl);
       blancs(bl); pariputs(s);        voir2((GEN)x[2],nb,bl);
@@ -1101,22 +1171,65 @@ get_var(long v, char *buf)
   sprintf(buf,"#<%d>",(int)v); return buf;
 }
 
+static void
+do_append(char **sp, char c, char *last, int count)
+{
+  if (*sp + count > last)
+    err(talker, "TeX variable name too long");
+  while (count--)
+    *(*sp)++ = c;
+}
+
 static char *
-get_texvar(long v, char *buf)
+get_texvar(long v, char *buf, unsigned int len)
 {
   entree *ep = varentries[v];
-  char *s, *t = buf;
+  char *s, *t = buf, *e = buf + len - 1;
 
   if (!ep) err(talker, "this object uses debugging variables");
   s = ep->name;
-  if (strlen(s)>=64) err(talker, "TeX variable name too long");
+  if (strlen(s) >= len) err(talker, "TeX variable name too long");
   while(isalpha((int)*s)) *t++ = *s++;
-  *t = 0; if (isdigit((int)*s) || *s++ == '_') sprintf(t,"_{%s}",s);
+  *t = 0;
+  if (isdigit((int)*s) || *s == '_') {
+    int seen1 = 0, seen = 0;
+
+    /* Skip until the first non-underscore */
+    while (*s == '_') s++, seen++;
+
+    /* Special-case integers and empty subscript */
+    if (*s == 0 || isdigit((unsigned char)*s))
+      seen++;
+
+    do_append(&t, '_', e, 1);
+    do_append(&t, '{', e, 1);
+    do_append(&t, '[', e, seen - 1);
+    while (1) {
+      if (*s == '_')
+        seen1++, s++;
+      else {
+        if (seen1) {
+          do_append(&t, ']', e, (seen >= seen1 ? seen1 : seen) - 1);
+          do_append(&t, ',', e, 1);
+          do_append(&t, '[', e, seen1 - 1);
+          if (seen1 > seen)
+            seen = seen1;
+          seen1 = 0;
+        }
+        if (*s == 0)
+          break;
+        do_append(&t, *s++, e, 1);
+      }
+    }
+    do_append(&t, ']', e, seen - 1);
+    do_append(&t, '}', e, 1);
+    *t = 0;
+  }
   return buf;
 }
 
 static void
-monome(char *v, long deg)
+monome(const char *v, long deg)
 {
   if (deg)
   {
@@ -1127,7 +1240,7 @@ monome(char *v, long deg)
 }
 
 static void
-texnome(char *v, long deg)
+texnome(const char *v, long deg)
 {
   if (deg)
   {
@@ -1144,7 +1257,7 @@ void
 etatpile(unsigned int n)
 {
   long nu, i, l, m;
-  gpmem_t av=avma;
+  pari_sp av=avma;
   GEN adr,adr1;
   double r;
 
@@ -1167,7 +1280,7 @@ etatpile(unsigned int n)
                  itos((GEN)adr[1]), itos((GEN)adr[2]));
   avma=av;
 
-  pariputsf(" %ld variable names used out of %d\n\n",manage_var(3,NULL),MAXVARN);
+  pariputsf(" %ld variable names used out of %d\n\n",manage_var(manage_var_next,NULL),MAXVARN);
   if (!n) return;
 
   if (n > (ulong)nu) n = nu;
@@ -1315,7 +1428,7 @@ isdenom(GEN g)
 
 /* write a * v^d */
 static void
-wr_monome(pariout_t *T, GEN a, char *v, long d)
+wr_monome(pariout_t *T, GEN a, const char *v, long d)
 {
   long sig = isone(a);
 
@@ -1333,7 +1446,7 @@ wr_monome(pariout_t *T, GEN a, char *v, long d)
 }
 
 static void
-wr_texnome(pariout_t *T, GEN a, char *v, long d)
+wr_texnome(pariout_t *T, GEN a, const char *v, long d)
 {
   long sig = isone(a);
 
@@ -1355,7 +1468,7 @@ wr_texnome(pariout_t *T, GEN a, char *v, long d)
 }
 
 static void
-wr_lead_monome(pariout_t *T, GEN a, char *v, long d, int nosign)
+wr_lead_monome(pariout_t *T, GEN a,const char *v, long d, int nosign)
 {
   long sig = isone(a);
   if (sig)
@@ -1375,7 +1488,7 @@ wr_lead_monome(pariout_t *T, GEN a, char *v, long d, int nosign)
 }
 
 static void
-wr_lead_texnome(pariout_t *T, GEN a, char *v, long d, int nosign)
+wr_lead_texnome(pariout_t *T, GEN a,const char *v, long d, int nosign)
 {
   long sig = isone(a);
   if (sig)
@@ -1403,7 +1516,8 @@ bruti(GEN g, pariout_t *T, int nosign)
 {
   long tg,l,i,j,r;
   GEN a,b;
-  char *v, buf[32];
+  const char *v;
+  char buf[32];
 
   if (!g) { pariputs("NULL"); return; }
   if (isnull(g)) { pariputc('0'); return; }
@@ -1418,9 +1532,7 @@ bruti(GEN g, pariout_t *T, int nosign)
   switch(tg)
   {
     case t_SMALL: pariputsf("%ld",smalltos(g)); break;
-    case t_INT:
-      if (!nosign && signe(g) < 0) pariputc('-');
-      wr_intpos(g); break;
+    case t_INT: wr_intsgn(g, !nosign && signe(g) < 0); break;
     case t_REAL: wr_real(T,g,nosign); break;
 
     case t_INTMOD: case t_POLMOD:
@@ -1478,9 +1590,10 @@ bruti(GEN g, pariout_t *T, int nosign)
     case t_PADIC:
     {
       GEN p = (GEN)g[2];
-      gpmem_t av = avma;
+      pari_sp av = avma;
+      char *ev;
       i = valp(g); l = precp(g)+i;
-      g = (GEN)g[4]; v = GENtostr(p);
+      g = (GEN)g[4]; ev = GENtostr(p);
       for (; i<l; i++)
       {
 	g = dvmdii(g,p,&a);
@@ -1488,15 +1601,15 @@ bruti(GEN g, pariout_t *T, int nosign)
 	{
 	  if (!i || !is_pm1(a))
 	  {
-	    wr_intpos(a); if (i) pariputc('*');
+	    wr_intsgn(a, 0); if (i) pariputc('*');
 	  }
-	  if (i) padic_nome(v,i);
+	  if (i) padic_nome(ev,i);
           sp_sign_sp(T,1);
 	}
         if ((i & 0xff) == 0) g = gerepileuptoint(av,g);
       }
-      pariputs("O("); padic_nome(v,i); pariputc(')');
-      free(v); break;
+      pariputs("O("); padic_nome(ev,i); pariputc(')');
+      free(ev); break;
     }
 
     case t_QFR: case t_QFI: r = (tg == t_QFR);
@@ -1582,7 +1695,7 @@ matbruti(GEN g, pariout_t *T)
 }
 
 static void
-sor_monome(pariout_t *T, GEN a, char *v, long d)
+sor_monome(pariout_t *T, GEN a, const char *v, long d)
 {
   long sig = isone(a);
   if (sig) { putsigne(sig); monome(v,d); }
@@ -1596,7 +1709,7 @@ sor_monome(pariout_t *T, GEN a, char *v, long d)
 }
 
 static void
-sor_lead_monome(pariout_t *T, GEN a, char *v, long d)
+sor_lead_monome(pariout_t *T, GEN a, const char *v, long d)
 {
   long sig = isone(a);
   if (sig)
@@ -1616,7 +1729,8 @@ sori(GEN g, pariout_t *T)
 {
   long tg=typ(g), i,j,r,l,close_paren;
   GEN a,b;
-  char *v, buf[32];
+  const char *v;
+  char buf[32];
 
   if (tg == t_INT) { wr_int(T,g,0); return; }
   if (tg != t_MAT && tg != t_COL) T->fieldw = 0;
@@ -1660,8 +1774,9 @@ sori(GEN g, pariout_t *T)
     case t_PADIC:
     {
       GEN p = (GEN)g[2];
+      char *ev;
       i = valp(g); l = precp(g)+i;
-      g = (GEN)g[4]; v = GENtostr(p);
+      g = (GEN)g[4]; ev = GENtostr(p);
       for (; i<l; i++)
       {
 	g = dvmdii(g,p,&a);
@@ -1671,13 +1786,13 @@ sori(GEN g, pariout_t *T)
 	  {
 	    wr_int(T,a,1); pariputc(i? '*': ' ');
 	  }
-	  if (i) { padic_nome(v,i); pariputc(' '); }
+	  if (i) { padic_nome(ev,i); pariputc(' '); }
           pariputs("+ ");
 	}
       }
       pariputs("O(");
-      if (!i) pariputs(" 1)"); else padic_nome(v,i);
-      pariputc(')'); free(v); break;
+      if (!i) pariputs(" 1)"); else padic_nome(ev,i);
+      pariputc(')'); free(ev); break;
     }
 
     case t_POL:
@@ -1796,7 +1911,8 @@ texi(GEN g, pariout_t *T, int nosign)
 {
   long tg,i,j,l,r;
   GEN a,b;
-  char *v, buf[67];
+  const char *v;
+  char buf[67];
 
   if (isnull(g)) { pariputs("{0}"); return; }
   r = isone(g); pariputc('{');
@@ -1810,9 +1926,7 @@ texi(GEN g, pariout_t *T, int nosign)
   switch(tg)
   {
     case t_SMALL: pariputsf("%ld",smalltos(g)); break;
-    case t_INT: 
-      if (!nosign && signe(g) < 0) pariputc('-');
-      wr_intpos(g); break;
+    case t_INT: wr_intsgn(g, !nosign && signe(g) < 0); break;
     case t_REAL: wr_real(T,g,nosign); break;
 
     case t_INTMOD: case t_POLMOD:
@@ -1834,7 +1948,7 @@ texi(GEN g, pariout_t *T, int nosign)
       if (!isnull(b)) wr_texnome(T,b,v,1);
       break;
 
-    case t_POL: v = get_texvar(ordvar[varn(g)],buf);
+    case t_POL: v = get_texvar(ordvar[varn(g)], buf, sizeof(buf));
       /* hack: we want g[i] = coeff of degree i. */
       i = degpol(g); g += 2; while (isnull((GEN)g[i])) i--;
       wr_lead_texnome(T,(GEN)g[i],v,i,nosign);
@@ -1845,7 +1959,7 @@ texi(GEN g, pariout_t *T, int nosign)
       }
       break;
 
-    case t_SER: v = get_texvar(ordvar[varn(g)],buf);
+    case t_SER: v = get_texvar(ordvar[varn(g)], buf, sizeof(buf));
       i = valp(g);
       if (signe(g))
       { /* hack: we want g[i] = coeff of degree i. */
@@ -1863,8 +1977,9 @@ texi(GEN g, pariout_t *T, int nosign)
     case t_PADIC:
     {
       GEN p = (GEN)g[2];
+      char *ev;
       i = valp(g); l = precp(g)+i;
-      g = (GEN)g[4]; v = GENtostr(p);
+      g = (GEN)g[4]; ev = GENtostr(p);
       for (; i<l; i++)
       {
 	g = dvmdii(g,p,&a);
@@ -1872,14 +1987,14 @@ texi(GEN g, pariout_t *T, int nosign)
 	{
 	  if (!i || !is_pm1(a))
 	  {
-	    wr_intpos(a); if (i) pariputs("\\cdot");
+	    wr_intsgn(a, 0); if (i) pariputs("\\cdot");
 	  }
-	  if (i) padic_texnome(v,i);
+	  if (i) padic_texnome(ev,i);
 	  pariputc('+');
 	}
       }
-      pariputs("O("); padic_texnome(v,i); pariputc(')');
-      free(v); break;
+      pariputs("O("); padic_texnome(ev,i); pariputc(')');
+      free(ev); break;
     }
     case t_QFR: case t_QFI: r = (tg == t_QFR);
       if (new_fun_set) pariputs("Qfb("); else pariputs(r? "qfr(": "qfi(");
@@ -1914,9 +2029,20 @@ texi(GEN g, pariout_t *T, int nosign)
       pariputc('}'); break;
 
     case t_STR:
-      pariputs("\\mbox{"); pariputs(GSTR(g));
+    {
+      char *s;
+  
+      pariputs("\\mbox{");
+      s = GSTR(g);
+      while (*s) {
+	if (strchr("\\{}$_^%#&~", *s))
+	  pariputc('\\');		/* What to do with \\ ? */
+	pariputc(*s);
+	if (strchr("^~", *s++))
+	  pariputs("{}");
+      }
       pariputc('}'); break;
-
+    }
     case t_MAT:
       pariputs("\\pmatrix{\n "); r = lg(g);
       if (r>1)
@@ -1978,6 +2104,7 @@ tex2mail_output(GEN z, long n)
 {
   pariout_t T = *(GP_DATA->fmt); /* copy */
   FILE *o_out;
+  FILE *o_logfile = logfile;
   
   if (!prettyp_init()) return 0;
   o_out = pari_outfile; /* save state */
@@ -1987,11 +2114,13 @@ tex2mail_output(GEN z, long n)
   pariflush();
   pari_outfile = GP_DATA->pp->file->file;
   T.prettyp = f_TEX;
+  logfile = NULL;
 
   /* history number */
   if (n)
   {
     char s[128];
+
     if (*term_get_color(c_HIST) || *term_get_color(c_OUTPUT))
     {
       char col1[80];
@@ -2002,12 +2131,21 @@ tex2mail_output(GEN z, long n)
     else
       sprintf(s, "\\%%%ld = ", n);
     pariputs_opt(s);
+    if (o_logfile)
+	fprintf(o_logfile, "%%%ld = ", n);
   }
   /* output */
   gen_output(z, &T);
+  
 
   /* flush and restore */
   prettyp_wait();
+  if (o_logfile) {
+    pari_outfile = o_logfile;
+    /* XXXX Maybe it is better to output in another format? */
+    outbrute(z); pariputc('\n'); pariflush();
+  }
+  logfile = o_logfile;
   pari_outfile = o_out;
   if (n) term_color(c_NONE);
   return 1;
@@ -2080,7 +2218,7 @@ gp_output(GEN z, gp_data *G)
 void
 gen_output(GEN x, pariout_t *T)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN y = changevar(x, polvar);
   if (!T) T = &DFLT_OUTPUT;
   T->initial = 1;
@@ -2186,7 +2324,7 @@ bruterr(GEN x,char format,long sigd)
 }
 
 void
-fprintferr(char* format, ...)
+fprintferr(const char* format, ...)
 {
   va_list args;
   PariOUT *out = pariOut; pariOut = pariErr;
@@ -2506,13 +2644,14 @@ pari_is_dir(char *name)
 
 /* expand tildes in filenames, return a malloc'ed buffer */
 static char *
-_expand_tilde(char *s)
+_expand_tilde(const char *s)
 {
 #if !defined(UNIX) && !defined(__EMX__)
   return pari_strdup(s);
 #else
   struct passwd *p;
-  char *u;
+  const char *u;
+  char *ret;
   int len;
 
   if (*s != '~') return pari_strdup(s);
@@ -2537,8 +2676,8 @@ _expand_tilde(char *s)
     p = getpwnam(tmp); free(tmp);
   }
   if (!p) err(talker2,"unknown user ",s,s-1);
-  s = gpmalloc(strlen(p->pw_dir) + strlen(u) + 1);
-  sprintf(s,"%s%s",p->pw_dir,u); return s;
+  ret = gpmalloc(strlen(p->pw_dir) + strlen(u) + 1);
+  sprintf(ret,"%s%s",p->pw_dir,u); return ret;
 #endif
 }
 
@@ -2601,7 +2740,7 @@ _expand_env(char *str)
 }
 
 char *
-expand_tilde(char *s)
+expand_tilde(const char *s)
 {
   return _expand_env(_expand_tilde(s));
 }
@@ -2705,7 +2844,7 @@ try_name(char *name)
 
 /* If name = "", re-read last file */
 void
-switchin(char *name0)
+switchin(const char *name0)
 {
   char *s, *name;
 
@@ -2780,12 +2919,12 @@ static long
 rd_long(FILE *f)
 {
   long L;
-  _lfread(&L, 1, f); return L;
+  _lfread(&L, 1UL, f); return L;
 }
 static void
 wr_long(long L, FILE *f)
 {
-  _lfwrite(&L, 1, f);
+  _lfwrite(&L, 1UL, f);
 }
 
 /* append x to file f */
@@ -2793,7 +2932,7 @@ static void
 wrGEN(GEN x, FILE *f)
 {
   GENbin *p = copy_bin(x);
-  long L = p->len;
+  size_t L = p->len;
 
   wr_long(L,f);
   wr_long((long)p->x,f);
@@ -2805,7 +2944,7 @@ wrGEN(GEN x, FILE *f)
 static void
 wrstr(char *s, FILE *f)
 {
-  long L = strlen(s)+1;
+  size_t L = strlen(s)+1;
   wr_long(L,f);
   _cfwrite(s, L, f);
 }
@@ -2813,7 +2952,7 @@ wrstr(char *s, FILE *f)
 static char *
 rdstr(FILE *f)
 {
-  long L = rd_long(f);
+  size_t L = (size_t)rd_long(f);
   char *s;
   if (!L) return NULL;
   s = gpmalloc(L);
@@ -2839,7 +2978,7 @@ writenamedGEN(GEN x, char *s, FILE *f)
 static GEN
 rdGEN(FILE *f)
 {
-  long L = rd_long(f);
+  size_t L = (size_t)rd_long(f);
   GENbin *p;
 
   if (!L) return NULL;
@@ -2889,7 +3028,7 @@ const long BINARY_VERSION = 0;
 static int
 is_magic_ok(FILE *f)
 {
-  int L = strlen(MAGIC);
+  size_t L = strlen(MAGIC);
   char *s = gpmalloc(L);
   int r = (fread(s,1,L, f) == L && strncmp(s,MAGIC,L) == 0);
   free(s); return r;
@@ -2910,7 +3049,7 @@ is_long_ok(FILE *f, long L)
 }
 
 static void
-check_magic(char *name, FILE *f)
+check_magic(const char *name, FILE *f)
 {
   if (!is_magic_ok(f))
     err(talker, "%s is not a GP binary file",name);
@@ -2936,7 +3075,7 @@ int
 file_is_binary(FILE *f)
 {
   int c = fgetc(f); ungetc(c,f);
-  return (isprint(c) == 0 && isspace(c) == 0);
+  return (c != EOF && isprint(c) == 0 && isspace(c) == 0);
 }
 
 void
@@ -2953,7 +3092,7 @@ writebin(char *name, GEN x)
   if (x) writeGEN(x,f);
   else
   {
-    long v, maxv = manage_var(3,NULL);
+    long v, maxv = manage_var(manage_var_next,NULL);
     for (v=0; v<maxv; v++)
     {
       entree *ep = varentries[v];
@@ -2968,9 +3107,9 @@ writebin(char *name, GEN x)
  * [i.e f contains more than one objet, not all of them 'named GENs'], return
  * them all in a vector with clone bit set (special marker). */
 GEN
-readbin(char *name, FILE *f)
+readbin(const char *name, FILE *f)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN x,y,z;
   int cx,cy;
   check_magic(name,f); x = y = z = NULL;
@@ -2997,31 +3136,32 @@ readbin(char *name, FILE *f)
 /**                             GP I/O                            **/
 /**                                                               **/
 /*******************************************************************/
-/* print a sequence of (NULL terminated) GEN */
+/* print a vector of GENs */
 void
-print0(GEN *g, long flag)
+print0(GEN g, long flag)
 {
   pariout_t T = GP_DATA? *(GP_DATA->fmt): DFLT_OUTPUT; /* copy */
+  long i, l = lg(g);
   T.prettyp = flag;
-  for( ; *g; g++)
-    if (typ(*g)==t_STR)
-      pariputs(GSTR(*g)); /* text surrounded by "" otherwise */
+  for (i = 1; i < l; i++)
+    if (flag != f_TEX && typ(g[i])==t_STR)
+      pariputs(GSTR(g[i])); /* text surrounded by "" otherwise */
     else
-      gen_output(*g, &T);
+      gen_output((GEN)g[i], &T);
 }
 
 #define PR_NL() {added_newline = 1; pariputc('\n'); pariflush(); }
 #define PR_NO() {added_newline = 0; pariflush(); }
-void print   (GEN *g) { print0(g, f_RAW);       PR_NL(); }
-void printp  (GEN *g) { print0(g, f_PRETTYOLD); PR_NL(); }
-void printtex(GEN *g) { print0(g, f_TEX);       PR_NL(); }
-void print1  (GEN *g) { print0(g, f_RAW);       PR_NO(); }
-void printp1 (GEN *g) { print0(g, f_PRETTYOLD); PR_NO(); }
+void print   (GEN g) { print0(g, f_RAW);       PR_NL(); }
+void printp  (GEN g) { print0(g, f_PRETTYOLD); PR_NL(); }
+void printtex(GEN g) { print0(g, f_TEX);       PR_NL(); }
+void print1  (GEN g) { print0(g, f_RAW);       PR_NO(); }
+void printp1 (GEN g) { print0(g, f_PRETTYOLD); PR_NO(); }
 
-void error0(GEN *g) { err(user, g); }
+void error0(GEN g) { err(user, g); }
 
 static char *
-wr_check(char *s) {
+wr_check(const char *s) {
   char *t = expand_tilde(s);
   if (GP_DATA && GP_DATA->flags & SECURE)
   {
@@ -3031,14 +3171,14 @@ wr_check(char *s) {
   return t;
 }
 
-static void wr_init(char *s)    { char *t=wr_check(s); switchout(t);   free(t);}
+static void wr_init(const char *s) { char *t=wr_check(s); switchout(t); free(t);}
 void gpwritebin(char *s, GEN x) { char *t=wr_check(s); writebin(t, x); free(t);}
 
 #define WR_NL() {pariputc('\n'); pariflush(); switchout(NULL); }
 #define WR_NO() {pariflush(); switchout(NULL); }
-void write0  (char *s, GEN *g) { wr_init(s); print0(g, f_RAW); WR_NL(); }
-void writetex(char *s, GEN *g) { wr_init(s); print0(g, f_TEX); WR_NL(); }
-void write1  (char *s, GEN *g) { wr_init(s); print0(g, f_RAW); WR_NO(); }
+void write0  (const char *s, GEN g) { wr_init(s); print0(g, f_RAW); WR_NL(); }
+void writetex(const char *s, GEN g) { wr_init(s); print0(g, f_TEX); WR_NL(); }
+void write1  (const char *s, GEN g) { wr_init(s); print0(g, f_RAW); WR_NO(); }
 
 /*******************************************************************/
 /**                                                               **/
