@@ -1,4 +1,4 @@
-/* $Id: sumiter.c,v 1.36 2002/07/15 13:30:01 karim Exp $
+/* $Id: sumiter.c,v 1.44 2003/01/06 22:03:43 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -32,7 +32,7 @@ extern GEN polint_i(GEN xa, GEN ya, GEN x, long n, GEN *ptdy);
 void
 forpari(entree *ep, GEN a, GEN b, char *ch)
 {
-  gpmem_t av, av0 = avma, lim;
+  pari_sp av, av0 = avma, lim;
 
   b = gcopy(b); av=avma; lim = stack_lim(av,1);
  /* gcopy nedeed in case b gets overwritten in ch, as in
@@ -41,7 +41,7 @@ forpari(entree *ep, GEN a, GEN b, char *ch)
   push_val(ep, a);
   while (gcmp(a,b) <= 0)
   {
-    gpmem_t av1=avma; (void)lisseq(ch); avma=av1;
+    pari_sp av1=avma; (void)lisseq(ch); avma=av1;
     if (loop_break()) break;
     a = (GEN) ep->value; a = gadd(a,gun);
     if (low_stack(lim, stack_lim(av,1)))
@@ -60,7 +60,7 @@ void
 forstep(entree *ep, GEN a, GEN b, GEN s, char *ch)
 {
   long ss, i;
-  gpmem_t av, av0 = avma, lim;
+  pari_sp av, av0 = avma, lim;
   GEN v = NULL;
   int (*cmp)(GEN,GEN);
 
@@ -77,7 +77,7 @@ forstep(entree *ep, GEN a, GEN b, GEN s, char *ch)
   i = 0;
   while (cmp(a,b) <= 0)
   {
-    gpmem_t av1=avma; (void)lisseq(ch); avma=av1;
+    pari_sp av1=avma; (void)lisseq(ch); avma=av1;
     if (loop_break()) break;
     if (v)
     {
@@ -99,12 +99,12 @@ forstep(entree *ep, GEN a, GEN b, GEN s, char *ch)
 /* assume ptr is the address of a diffptr containing the succesive
  * differences between primes, and c = current prime (up to *p excluded)
  * return smallest prime >= a, update ptr */
-static long
-sinitp(long a, long c, byteptr *ptr)
+static ulong
+sinitp(ulong a, ulong c, byteptr *ptr)
 {
   byteptr p = *ptr;
   if (a <= 0) a = 2;
-  if (maxprime() < (ulong)a) err(primer1);
+  maxprime_check((ulong)a);
   while (a > c)
      NEXT_PRIME_VIADIFF(c,p);
   *ptr = p; return c;
@@ -113,14 +113,14 @@ sinitp(long a, long c, byteptr *ptr)
 /* value changed during the loop, replace by the first prime whose
    value is strictly larger than new value */
 static void
-update_p(entree *ep, byteptr *ptr, long prime[])
+update_p(entree *ep, byteptr *ptr, ulong prime[])
 {
   GEN z = (GEN)ep->value;
-  long a, c;
+  ulong a, c;
 
   if (typ(z) == t_INT) a = 1; else { z = gceil(z); a = 0; }
-  if (is_bigint(z)) { prime[2] = -1; /* = infinity */ return; }
-  a += itos(z); c = prime[2];
+  if (lgefint(z) > 3) { prime[2] = (long)MAXULONG; /* = infinity */ return; }
+  a += itou(z); c = prime[2];
   if (c < a)
     prime[2] = sinitp(a, c, ptr); /* increased */
   else if (c > a)
@@ -128,53 +128,54 @@ update_p(entree *ep, byteptr *ptr, long prime[])
     *ptr = diffptr;
     prime[2] = sinitp(a, 0, ptr);
   }
-  changevalue_p(ep, prime);
+  changevalue_p(ep, (GEN)prime);
 }
 
 static byteptr
-prime_loop_init(GEN ga, GEN gb, long *a, long *b, long prime[])
+prime_loop_init(GEN ga, GEN gb, ulong *a, ulong *b, ulong prime[])
 {
-  byteptr p = diffptr;
-  ulong P;
+  byteptr d = diffptr;
 
   ga = gceil(ga); gb = gfloor(gb);
   if (typ(ga) != t_INT || typ(gb) != t_INT)
     err(typeer,"prime_loop_init");
-  if (is_bigint(ga) || is_bigint(gb))
+  if (signe(gb) < 0) return NULL;
+  if (signe(ga) < 0) ga = gun;
+  if (lgefint(ga)>3 || lgefint(gb)>3)
   {
     if (cmpii(ga, gb) > 0) return NULL;
     err(primer1);
   }
-  P = maxprime();
-  *a = itos(ga); if (*a <= 0) *a = 1;
-  *b = itos(gb); if (*a > *b) return NULL;
-  if ((ulong)*a <= P) prime[2] = sinitp(*a, 0, &p);
-  if ((ulong)*b > P) err(primer1);
-  return p;
+  *a = itou(ga);
+  *b = itou(gb); if (*a > *b) return NULL;
+  maxprime_check(*b);
+  prime[2] = sinitp(*a, 0, &d);
+  return d;
 }
 
 void
 forprime(entree *ep, GEN ga, GEN gb, char *ch)
 {
-  long prime[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
-  long a, b;
-  gpmem_t av = avma;
-  byteptr p;
+  long p[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
+  ulong *prime = (ulong*)p;
+  ulong a, b;
+  pari_sp av = avma;
+  byteptr d;
 
-  p = prime_loop_init(ga,gb, &a,&b, prime);
-  if (!p) { avma = av; return; }
+  d = prime_loop_init(ga,gb, &a,&b, prime);
+  if (!d) { avma = av; return; }
 
-  avma = av; push_val(ep, prime);
-  while ((ulong)prime[2] < (ulong)b)
+  avma = av; push_val(ep, (GEN)prime);
+  while (prime[2] < b)
   {
     (void)lisseq(ch); if (loop_break()) break;
     if (ep->value == prime)
-      prime[2] += *p++;
+      NEXT_PRIME_VIADIFF(prime[2], d);
     else
-      update_p(ep, &p, prime);
+      update_p(ep, &d, prime);
     avma = av;
   }
-  /* if b = P --> *p = 0 now and the loop wouldn't end if it read 'while
+  /* if b = P --> *d = 0 now and the loop wouldn't end if it read 'while
    * (prime[2] <= b)' */
   if (prime[2] == b) { (void)lisseq(ch); (void)loop_break(); avma = av; }
   pop_val(ep);
@@ -184,7 +185,7 @@ void
 fordiv(GEN a, entree *ep, char *ch)
 {
   long i, l;
-  gpmem_t av2, av = avma;
+  pari_sp av2, av = avma;
   GEN t = divisors(a);
 
   push_val(ep, NULL); l=lg(t); av2 = avma;
@@ -225,14 +226,14 @@ fvloop(long i, fvdat *d)
   if (i+1 == d->n)
     while (gcmp(d->a[i], d->M[i]) <= 0)
     {
-      gpmem_t av = avma; (void)lisseq(d->ch); avma = av;
+      pari_sp av = avma; (void)lisseq(d->ch); avma = av;
       if (loop_break()) { d->n = 0; return; }
       d->a[i] = gadd(d->a[i], gun);
     }
   else
     while (gcmp(d->a[i], d->M[i]) <= 0)
     {
-      gpmem_t av = avma; fvloop(i+1, d); avma = av;
+      pari_sp av = avma; fvloop(i+1, d); avma = av;
       if (!d->n) return;
       d->a[i] = gadd(d->a[i], gun);
     }
@@ -257,14 +258,14 @@ fvloop_i(long i, fvdat *d)
   if (i+1 == d->n)
     while (gcmp(d->a[i], d->M[i]) <= 0)
     {
-      gpmem_t av = avma; (void)lisseq(d->ch); avma = av;
+      pari_sp av = avma; (void)lisseq(d->ch); avma = av;
       if (loop_break()) { d->n = 0; return; }
       d->a[i] = incloop(d->a[i]);
     }
   else
     while (gcmp(d->a[i], d->M[i]) <= 0)
     {
-      gpmem_t av = avma; fvloop_i(i+1, d); avma = av;
+      pari_sp av = avma; fvloop_i(i+1, d); avma = av;
       if (!d->n) return;
       d->a[i] = incloop(d->a[i]);
     }
@@ -273,7 +274,7 @@ fvloop_i(long i, fvdat *d)
 void
 forvec(entree *ep, GEN x, char *c, long flag)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long tx = typ(x);
   fvdat D, *d = &D;
 
@@ -315,7 +316,7 @@ forvec(entree *ep, GEN x, char *c, long flag)
 GEN
 somme(entree *ep, GEN a, GEN b, char *ch, GEN x)
 {
-  gpmem_t av, av0 = avma, lim;
+  pari_sp av, av0 = avma, lim;
   GEN p1;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sum");
@@ -345,7 +346,7 @@ GEN
 suminf(entree *ep, GEN a, char *ch, long prec)
 {
   long fl, G;
-  gpmem_t tetpil, av0 = avma, av, lim;
+  pari_sp tetpil, av0 = avma, av, lim;
   GEN p1,x = realun(prec);
 
   if (typ(a) != t_INT) err(talker,"non integral index in suminf");
@@ -375,7 +376,7 @@ suminf(entree *ep, GEN a, char *ch, long prec)
 GEN
 divsum(GEN num, entree *ep, char *ch)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN z, y = gzero, t = divisors(num);
   long i, l = lg(t);
 
@@ -399,7 +400,7 @@ divsum(GEN num, entree *ep, char *ch)
 GEN
 produit(entree *ep, GEN a, GEN b, char *ch, GEN x)
 {
-  gpmem_t av, av0 = avma, lim;
+  pari_sp av, av0 = avma, lim;
   GEN p1;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sum");
@@ -440,7 +441,7 @@ prodinf0(entree *ep, GEN a, char *ch, long flag, long prec)
 GEN
 prodinf(entree *ep, GEN a, char *ch, long prec)
 {
-  gpmem_t av0 = avma, av, lim;
+  pari_sp av0 = avma, av, lim;
   long fl,G;
   GEN p1,x = realun(prec);
 
@@ -467,7 +468,7 @@ prodinf(entree *ep, GEN a, char *ch, long prec)
 GEN
 prodinf1(entree *ep, GEN a, char *ch, long prec)
 {
-  gpmem_t av0 = avma, av, lim;
+  pari_sp av0 = avma, av, lim;
   long fl,G;
   GEN p1,p2,x = realun(prec);
 
@@ -494,19 +495,19 @@ prodinf1(entree *ep, GEN a, char *ch, long prec)
 GEN
 prodeuler(entree *ep, GEN ga, GEN gb, char *ch, long prec)
 {
-  long prime[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
-  long a,b;
-  gpmem_t av, av0 = avma, lim;
+  long p[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
+  ulong a,b, *prime = (ulong*)p;
+  pari_sp av, av0 = avma, lim;
   GEN p1,x = realun(prec);
-  byteptr p;
+  byteptr d;
 
   av = avma;
-  p = prime_loop_init(ga,gb, &a,&b, prime);
-  if (!p) { avma = av; return x; }
+  d = prime_loop_init(ga,gb, &a,&b, prime);
+  if (!d) { avma = av; return x; }
 
-  av = avma; push_val(ep, prime);
+  av = avma; push_val(ep, (GEN)prime);
   lim = stack_lim(avma,1);
-  while ((ulong)prime[2] < (ulong)b)
+  while (prime[2] < b)
   {
     p1 = lisexpr(ch); if (did_break()) err(breaker,"prodeuler");
     x = gmul(x,p1);
@@ -516,9 +517,9 @@ prodeuler(entree *ep, GEN ga, GEN gb, char *ch, long prec)
       x = gerepilecopy(av, x);
     }
     if (ep->value == prime)
-      prime[2] += *p++;
+      NEXT_PRIME_VIADIFF(prime[2], d);
     else
-      update_p(ep, &p, prime);
+      update_p(ep, &d, prime);
   }
   /* cf forprime */
   if (prime[2] == b)
@@ -532,17 +533,18 @@ prodeuler(entree *ep, GEN ga, GEN gb, char *ch, long prec)
 GEN
 direulerall(entree *ep, GEN ga, GEN gb, char *ch, GEN c)
 {
-  long prime[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
-  gpmem_t av0 = avma, av, lim = stack_lim(av0, 1);
-  long p,n,i,j,k,tx,lx,a,b;
-  GEN x,y,s,polnum,polden;
+  long pp[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
+  ulong a, b, i, k, n, p, *prime = (ulong*)pp;
+  pari_sp av0 = avma, av, lim = stack_lim(av0, 1);
+  long j, tx, lx;
+  GEN x, y, s, polnum, polden;
   byteptr d;
 
   d = prime_loop_init(ga,gb, &a,&b, prime);
-  n = c? itos(c): b;
-  if (!d || b < 2 || n <= 0) return _vec(gun);
+  n = c? itou(c): b;
+  if (!d || b < 2 || (c && signe(c) < 0)) return _vec(gun);
   if (n < b) b = n;
-  push_val(ep, prime);
+  push_val(ep, (GEN)prime);
 
   y = cgetg(n+1,t_VEC); av = avma;
   x = cgetg(n+1,t_VEC); x[1]=un; for (i=2; i<=n; i++) x[i]=zero;
@@ -564,7 +566,7 @@ direulerall(entree *ep, GEN ga, GEN gb, char *ch, GEN c)
     }
     else
     {
-      ulong k1,q, qlim;
+      ulong k1, q, qlim;
       if (tx != t_POL) err(typeer,"direuler");
       c = truecoeff(polnum,0);
       if (!gcmp1(c))
@@ -576,12 +578,12 @@ direulerall(entree *ep, GEN ga, GEN gb, char *ch, GEN c)
       }
       for (i=1; i<=n; i++) y[i]=x[i];
       lx=degpol(polnum);
-      q=p; qlim = n/p; j=1;
-      while (q<=(ulong)n && j<=lx)
+      q = p; qlim = n/p; j=1;
+      while (q<=n && j<=lx)
       {
 	c=(GEN)polnum[j+2];
 	if (!gcmp0(c))
-	  for (k=1,k1=q; k1<=(ulong)n; k1+=q,k++)
+	  for (k=1,k1=q; k1<=n; k1+=q,k++)
 	    x[k1] = ladd((GEN)x[k1], gmul(c,(GEN)y[k]));
         if (q > qlim) break;
 	q*=p; j++;
@@ -615,7 +617,7 @@ direulerall(entree *ep, GEN ga, GEN gb, char *ch, GEN c)
       if (DEBUGMEM>1) err(warnmem,"direuler");
       x = gerepilecopy(av, x);
     }
-    p += *d++; if (!*d) err(primer1);
+    NEXT_PRIME_VIADIFF(p, d);
     prime[2] = p;
   }
   pop_val(ep); return gerepilecopy(av0,x);
@@ -772,7 +774,7 @@ GEN
 sumalt(entree *ep, GEN a, char *ch, long prec)
 {
   long k, N;
-  gpmem_t av = avma, tetpil;
+  pari_sp av = avma, tetpil;
   GEN s,az,c,x,e1,d;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sumalt");
@@ -800,7 +802,7 @@ GEN
 sumalt2(entree *ep, GEN a, char *ch, long prec)
 {
   long k, N;
-  gpmem_t av = avma, tetpil;
+  pari_sp av = avma, tetpil;
   GEN x,s,dn,pol;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sumalt");
@@ -828,7 +830,7 @@ GEN
 sumpos(entree *ep, GEN a, char *ch, long prec)
 {
   long k, kk, N, G;
-  gpmem_t av = avma, tetpil;
+  pari_sp av = avma, tetpil;
   GEN p1,r,q1,reel,s,az,c,x,e1,d, *stock;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sumpos");
@@ -874,7 +876,7 @@ GEN
 sumpos2(entree *ep, GEN a, char *ch, long prec)
 {
   long k, kk, N, G;
-  gpmem_t av = avma, tetpil;
+  pari_sp av = avma, tetpil;
   GEN p1,r,q1,reel,s,pol,dn,x, *stock;
 
   if (typ(a) != t_INT) err(talker,"non integral index in sumpos2");
@@ -946,7 +948,7 @@ fix(GEN a, long prec)
 GEN
 int_loop(entree *ep, char *ch)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   intstruct T;
 
   T.in = NULL;
@@ -981,7 +983,7 @@ _invf(void *dat, GEN x)
 static GEN
 interp(GEN h, GEN s, long j, long lim, long KLOC)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   long e1,e2;
   GEN dss, ss = polint_i(h+j-KLOC,s+j-KLOC,gzero,KLOC+1,&dss);
 
@@ -998,7 +1000,7 @@ qrom3(void *dat, GEN (*eval)(void *, GEN), GEN a, GEN b, long prec)
   const long JMAX = 25, KLOC = 4;
   GEN ss,s,h,p1,p2,qlint,del,x,sum;
   long j, j1, it, sig;
-  gpmem_t av;
+  pari_sp av;
 
   a = fix(a,prec);
   b = fix(b,prec);
@@ -1035,7 +1037,7 @@ qrom2(void *dat, GEN (*eval)(void *, GEN), GEN a, GEN b, long prec)
   const long JMAX = 16, KLOC = 4;
   GEN ss,s,h,p1,qlint,del,ddel,x,sum;
   long j, j1, it, sig;
-  gpmem_t av;
+  pari_sp av;
 
   a = fix(a, prec);
   b = fix(b, prec);
@@ -1117,7 +1119,7 @@ rombint(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, long prec)
 GEN
 intnum(void *E, GEN (*eval)(void*,GEN), GEN a, GEN b, long flag, long prec)
 {
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN z;
   switch(flag)
   {
@@ -1151,34 +1153,19 @@ intnum0(entree *ep, GEN a, GEN b, char *ch, long flag, long prec)
 GEN
 polzag(long n, long m)
 {
-  gpmem_t av = avma;
-  long d1, d, r, k;
-  GEN A, B, Bx, g, s;
+  pari_sp av = avma;
+  long k, d = n - m;
+  GEN A, Bx, g, s;
 
-  if (m >= n || m < 0)
-    err(talker,"first index must be greater than second in polzag");
-  d1 = n-m; d = d1<<1; d1--;
-  A = gsub(gun, gmul2n(polx[0],1)); /* 1 - 2x */
-  B = gsub(gun, polx[0]); /* 1 - x */
-  Bx = gmul(polx[0],B); /* x - x^2 */
-  r = (m+1)>>1;
-  g = gzero;
-  for (k=0; k<=d1; k++)
-  {
-    s = binome(stoi(d), (k<<1)+1); if (k&1) s = negi(s);
-    g = gadd(g, gmul(s, gmul(gpowgs(polx[0],k), gpowgs(B,d1-k))));
-  }
-  g = gmul(g, gpowgs(Bx,r));
-  if (!(m&1)) g = gadd(gmul(A,g), gmul2n(gmul(Bx,derivpol(g)),1));
-  for (k=1; k<=r; k++)
-  {
-    g = derivpol(g);
-    g = gadd(gmul(A,g), gmul2n(gmul(Bx,derivpol(g)),1));
-  }
-  g = m? gmul2n(g,(m-1)>>1): gmul2n(g,-1);
-  s = mulsi(n-m, mpfact(m+1));
+  if (d <= 0 || m < 0) return gzero;
+  A  = gsub(gun, gmul2n(polx[0],1)); /* 1 - 2x */
+  Bx = gmul2n( gmul(polx[0], gsub(gun, polx[0])), 1); /* 2x - 2x^2 */
+  g = gmul(poleval(derivpol(tchebi(d,0)), A), gpowgs(Bx, (m+1)>>1));
+  for (k = m; k >= 0; k--)
+    g = (k&1)? derivpol(g): gadd(gmul(A,g), gmul(Bx,derivpol(g)));
+  s = mulsi(d, mulsi(d, mpfact(m+1)));
   return gerepileupto(av, gdiv(g,s));
-}
+} 
 
 #ifdef _MSC_VER /* Bill Daly: work around a MSVC bug */
 #pragma optimize("g",off)
@@ -1186,30 +1173,28 @@ polzag(long n, long m)
 GEN
 polzagreel(long n, long m, long prec)
 {
-  long d1, d, r, j, k, k2;
-  gpmem_t av = avma;
-  GEN Bx,B,g,h,v,b,s;
+  long r, j, k, k2, d = n - m, d2 = d<<1;
+  pari_sp av = avma;
+  GEN Bx, g, h, v, b, s;
 
-  if (m >= n || m < 0)
-    err(talker,"first index must be greater than second in polzag");
-  d1 = n-m; d = d1<<1; d1--;
-  B = gadd(gun,polx[0]);
-  Bx = gmul(polx[0],B);
-  r = (m+1)>>1;
-  v = cgetg(d1+2,t_VEC);
-  g = cgetg(d1+2,t_VEC);
-  v[d1+1] = un; b = stor(d, prec);
-  g[d1+1] = (long)b;
-  for (k=1; k<=d1; k++)
+  if (d <= 0 || m < 0) return gzero;
+  Bx = gmul(polx[0], gadd(gun, polx[0])); /* x + x^2 */
+  v = cgetg(d+1,t_VEC);
+  g = cgetg(d+1,t_VEC);
+  v[d] = un; b = stor(d2, prec);
+  g[d] = (long)b;
+  for (k = 1; k < d; k++)
   {
-    v[d1-k+1] = un;
+    v[d-k] = un;
     for (j=1; j<k; j++)
-      v[d1-k+j+1] = laddii((GEN)v[d1-k+j+1], (GEN)v[d1-k+j+2]);
-    k2 = k+k; b = divri(mulri(b,mulss(d-k2+1,d-k2)), mulss(k2,k2+1));
+      v[d-k+j] = laddii((GEN)v[d-k+j], (GEN)v[d-k+j+1]);
+    /* v[d-k+j] = binom(k, j), j = 0..k */
+    k2 = k+k; b = divri(mulri(b,mulss(d2-k2+1,d2-k2)), mulss(k2,k2+1));
     for (j=1; j<=k; j++)
-      g[d1-k+j+1] = lmpadd((GEN)g[d1-k+j+1], mulri(b,(GEN)v[d1-k+j+1]));
-    g[d1-k+1] = (long)b;
+      g[d-k+j] = lmpadd((GEN)g[d-k+j], mulri(b,(GEN)v[d-k+j]));
+    g[d-k] = (long)b;
   }
+  r = (m+1)>>1;
   g = gmul(vec_to_pol(g,0), gpowgs(Bx,r));
   for (j=0; j<=r; j++)
   {
@@ -1226,7 +1211,7 @@ polzagreel(long n, long m, long prec)
     }
   }
   g = m? gmul2n(g,(m-1)>>1): gmul2n(g,-1);
-  s = mulsi(n-m, mpfact(m+1));
+  s = mulsi(d, mpfact(m+1));
   return gerepileupto(av, gdiv(g,s));
 }
 #ifdef _MSC_VER
@@ -1243,7 +1228,7 @@ GEN
 zbrent(entree *ep, GEN a, GEN b, char *ch, long prec)
 {
   long sig, iter, itmax;
-  gpmem_t av = avma;
+  pari_sp av = avma;
   GEN c,d,e,tol,tol1,min1,min2,fa,fb,fc,p,q,r,s,xm;
 
   a = fix(a,prec);
@@ -1256,7 +1241,7 @@ zbrent(entree *ep, GEN a, GEN b, char *ch, long prec)
   if (gsigne(fa)*gsigne(fb) > 0)
     err(talker,"roots must be bracketed in solve");
   itmax = (prec<<(TWOPOTBITS_IN_LONG+1)) + 1;
-  tol = realun(3); setexpo(tol, 5-bit_accuracy(prec));
+  tol = real2n(5-bit_accuracy(prec), 3);
   fc=fb;
   e = d = NULL; /* gcc -Wall */
   for (iter=1; iter<=itmax; iter++)
